@@ -85,7 +85,7 @@ static int _load_config(void)
 	tbl = s_p_hashtbl_create(options);
 	if (stat(conf, &statbuf) || !statbuf.st_size) {
 		/* No file, continue below to set defaults */
-	} else if (s_p_parse_file(tbl, NULL, conf, false, NULL)) {
+	} else if (s_p_parse_file(tbl, NULL, conf, false, NULL, false)) {
 		/* could not load or parse file */
 		return (config_loaded = SLURM_ERROR);
 	}
@@ -171,7 +171,7 @@ static struct hostent *_host_internal(int mode, const char *nodename)
 }
 
 static int _internal_gethost(int af, const char *name, char *buf,
-			     size_t buflen, struct hostent *result)
+			     size_t buflen, struct hostent *result, int *errnop)
 {
 	int len_name, len_aliases = 0, cnt_aliases = 0, len_addr;
 	struct hostent *rpc_result = NULL;
@@ -198,7 +198,8 @@ static int _internal_gethost(int af, const char *name, char *buf,
 	if ((len_name + len_aliases + ((cnt_aliases + 1) * sizeof(char*)) +
 	     len_addr + (2 * sizeof(char *)) + cnt_aliases + 1) > buflen) {
 		xfree_struct_hostent(rpc_result);
-		return ERANGE;
+		*errnop = ERANGE;
+		return NSS_STATUS_TRYAGAIN;
 	}
 
 	strncpy(buf, rpc_result->h_name, len_name + 1);
@@ -234,7 +235,7 @@ enum nss_status _nss_slurm_gethostbyname_r(const char *name,
 					   size_t buflen, int *errnop,
 					   int *herrnop)
 {
-	return _internal_gethost(AF_UNSPEC, name, buf, buflen, result);
+	return _internal_gethost(AF_UNSPEC, name, buf, buflen, result, errnop);
 }
 
 enum nss_status _nss_slurm_gethostbyname2_r(const char *name, int af,
@@ -242,7 +243,7 @@ enum nss_status _nss_slurm_gethostbyname2_r(const char *name, int af,
 					    char *buf, size_t buflen,
 					    int *errnop, int *herrnop)
 {
-	return _internal_gethost(af, name, buf, buflen, result);
+	return _internal_gethost(af, name, buf, buflen, result, errnop);
 }
 
 static struct passwd *_pw_internal(int mode, uid_t uid, const char *name)
@@ -285,7 +286,7 @@ static struct passwd *_pw_internal(int mode, uid_t uid, const char *name)
 
 static int _internal_getpw(int mode, uid_t uid, const char *name,
 			   struct passwd *pwd, char *buf, size_t buflen,
-			   struct passwd **result)
+			   int *errnop)
 {
 	int len_name, len_passwd, len_gecos, len_dir, len_shell;
 	struct passwd *rpc_result = NULL;
@@ -303,7 +304,8 @@ static int _internal_getpw(int mode, uid_t uid, const char *name,
 	if ((len_name + len_passwd + len_gecos + len_dir + len_shell + 5)
 	    > buflen) {
 		xfree_struct_passwd(rpc_result);
-		return ERANGE;
+		*errnop = ERANGE;
+		return NSS_STATUS_TRYAGAIN;
 	}
 
 	strncpy(buf, rpc_result->pw_name, len_name + 1);
@@ -328,24 +330,22 @@ static int _internal_getpw(int mode, uid_t uid, const char *name,
 	strncpy(buf, rpc_result->pw_shell, len_shell + 1);
 	pwd->pw_shell = buf;
 
-	*result = pwd;
 	xfree_struct_passwd(rpc_result);
 	return NSS_STATUS_SUCCESS;
 }
 
-int _nss_slurm_getpwnam_r(const char *name, struct passwd *pwd,
-			  char *buf, size_t buflen, struct passwd **result)
+enum nss_status _nss_slurm_getpwnam_r(const char *name, struct passwd *pwd,
+				      char *buf, size_t buflen, int *errnop)
 {
 	return _internal_getpw(GETPW_MATCH_USER_AND_PID, NO_VAL, name, pwd,
-			       buf, buflen, result);
-
+			       buf, buflen, errnop);
 }
 
-int _nss_slurm_getpwuid_r(uid_t uid, struct passwd *pwd,
-			  char *buf, size_t buflen, struct passwd **result)
+enum nss_status _nss_slurm_getpwuid_r(uid_t uid, struct passwd *pwd,
+				      char *buf, size_t buflen, int *errnop)
 {
 	return _internal_getpw(GETPW_MATCH_USER_AND_PID, uid, NULL, pwd,
-			       buf, buflen, result);
+			       buf, buflen, errnop);
 }
 
 static int entry_fetched = 1;
@@ -356,8 +356,8 @@ int _nss_slurm_setpwent(void)
 	return NSS_STATUS_SUCCESS;
 }
 
-int _nss_slurm_getpwent_r(struct passwd *pwd, char *buf, size_t buflen,
-			  struct passwd **result)
+enum nss_status _nss_slurm_getpwent_r(struct passwd *pwd, char *buf,
+				      size_t buflen, int *errnop)
 {
 	/*
 	 * There is only ever one entry here. The docs indicate we should
@@ -367,11 +367,11 @@ int _nss_slurm_getpwent_r(struct passwd *pwd, char *buf, size_t buflen,
 		return NSS_STATUS_NOTFOUND;
 	entry_fetched = 1;
 
-	return _internal_getpw(GETPW_MATCH_PID, NO_VAL, NULL, pwd,
-			     buf, buflen, result);
+	return _internal_getpw(GETPW_MATCH_PID, NO_VAL, NULL, pwd, buf, buflen,
+			       errnop);
 }
 
-int _nss_slurm_endpwent(void)
+enum nss_status _nss_slurm_endpwent(void)
 {
 	return NSS_STATUS_SUCCESS;
 }
@@ -420,7 +420,7 @@ static struct group **gr_rpc_results = NULL;
 
 static int _internal_getgr(int mode, gid_t gid, const char *name,
 			   struct group *grp, char *buf, size_t buflen,
-			   struct group **result)
+			   int *errnop)
 {
 	int len_name, len_passwd, len_mem = 0;
 	int i = 0;
@@ -458,7 +458,8 @@ static int _internal_getgr(int mode, gid_t gid, const char *name,
 	if ((len_name + len_passwd + len_mem + 3)
 	    > buflen) {
 		xfree_struct_group_array(gr_rpc_results);
-		return ERANGE;
+		*errnop = ERANGE;
+		return NSS_STATUS_TRYAGAIN;
 	}
 
 	strncpy(buf, gr_rpc_results[i]->gr_name, len_name + 1);
@@ -499,7 +500,6 @@ static int _internal_getgr(int mode, gid_t gid, const char *name,
 	} else
 		grp->gr_mem = NULL;
 
-	*result = grp;
 	if (mode != GETGR_MATCH_PID) {
 		xfree_struct_group_array(gr_rpc_results);
 		gr_rpc_results = NULL;
@@ -513,22 +513,22 @@ static int _internal_getgr(int mode, gid_t gid, const char *name,
 	return NSS_STATUS_SUCCESS;
 }
 
-int _nss_slurm_getgrnam_r(const char *name, struct group *pwd,
-			  char *buf, size_t buflen, struct group **result)
+enum nss_status _nss_slurm_getgrnam_r(const char *name, struct group *pwd,
+				      char *buf, size_t buflen, int *errnop)
 {
 	return _internal_getgr(GETGR_MATCH_GROUP_AND_PID, NO_VAL, name, pwd,
-			       buf, buflen, result);
+			       buf, buflen, errnop);
 
 }
 
-int _nss_slurm_getgrgid_r(gid_t gid, struct group *pwd,
-			  char *buf, size_t buflen, struct group **result)
+enum nss_status _nss_slurm_getgrgid_r(gid_t gid, struct group *pwd,
+				      char *buf, size_t buflen, int *errnop)
 {
 	return _internal_getgr(GETGR_MATCH_GROUP_AND_PID, gid, NULL, pwd,
-			       buf, buflen, result);
+			       buf, buflen, errnop);
 }
 
-int _nss_slurm_setgrent(void)
+enum nss_status _nss_slurm_setgrent(void)
 {
 	xfree_struct_group_array(gr_rpc_results);
 	gr_rpc_results = NULL;
@@ -537,14 +537,14 @@ int _nss_slurm_setgrent(void)
 	return NSS_STATUS_SUCCESS;
 }
 
-int _nss_slurm_getgrent_r(struct group *grp, char *buf, size_t buflen,
-			  struct group **result)
+enum nss_status _nss_slurm_getgrent_r(struct group *grp, char *buf,
+				      size_t buflen, int *errnop)
 {
-	return _internal_getgr(GETGR_MATCH_PID, NO_VAL, NULL, grp,
-			       buf, buflen, result);
+	return _internal_getgr(GETGR_MATCH_PID, NO_VAL, NULL, grp, buf, buflen,
+			       errnop);
 }
 
-int _nss_slurm_endgrent(void)
+enum nss_status _nss_slurm_endgrent(void)
 {
 	xfree_struct_group_array(gr_rpc_results);
 	gr_rpc_results = NULL;
