@@ -57,6 +57,7 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 #include "src/common/read_config.h"
+#include "src/common/slurm_resource_info.h"
 #include "src/slurmd/slurmd/slurmd.h"
 
 #define PATH_TO_CPU	"/sys/devices/system/cpu/"
@@ -69,10 +70,9 @@
 #define GOV_PERFORMANCE		0x04
 #define GOV_POWERSAVE		0x08
 #define GOV_USERSPACE		0x10
+#define GOV_SCHEDUTIL		0x20
 
 static uint16_t cpu_freq_count = 0;
-static uint32_t cpu_freq_govs = 0; /* Governors allowed. */
-static uint64_t debug_flags = NO_VAL; /* init value for slurmd, slurmstepd */
 static int set_batch_freq = -1;
 
 static struct cpu_freq_data {
@@ -97,7 +97,7 @@ static uint16_t	_cpu_freq_next_cpu(char **core_range, uint16_t *cpx,
 				   uint16_t *start, uint16_t *end);
 static uint32_t	_cpu_freq_get_scaling_freq(int cpuidx, char* option);
 static void     _cpu_freq_init_data(int cpx);
-static void     _cpu_freq_setup_data(stepd_step_rec_t *job, int cpx);
+static void     _cpu_freq_setup_data(stepd_step_rec_t *step, int cpx);
 static bool	_cpu_freq_test_scaling_freq(int cpuidx, char *option);
 static int	_derive_avail_freq(int cpuidx);
 static int	_fd_lock_retry(int fd);
@@ -286,8 +286,6 @@ cpu_freq_init(slurmd_conf_t *conf)
 	char value[LINE_LEN];
 	unsigned int i, j;
 
-	debug_flags = slurm_get_debug_flags(); /* init for slurmd */
-
 	xfree(slurmd_spooldir);
 	slurmd_spooldir = xstrdup(conf->spooldir);
 
@@ -331,43 +329,39 @@ cpu_freq_init(slurmd_conf_t *conf)
 		}
 		if (strstr(value, "conservative")) {
 			cpufreq[i].avail_governors |= GOV_CONSERVATIVE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: Conservative governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: Conservative governor defined on cpu 0");
 		}
 		if (strstr(value, "ondemand")) {
 			cpufreq[i].avail_governors |= GOV_ONDEMAND;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: OnDemand governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: OnDemand governor defined on cpu 0");
 		}
 		if (strstr(value, "performance")) {
 			cpufreq[i].avail_governors |= GOV_PERFORMANCE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: Performance governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: Performance governor defined on cpu 0");
 		}
 		if (strstr(value, "powersave")) {
 			cpufreq[i].avail_governors |= GOV_POWERSAVE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: PowerSave governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: PowerSave governor defined on cpu 0");
 		}
 		if (strstr(value, "userspace")) {
 			cpufreq[i].avail_governors |= GOV_USERSPACE;
-			if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
-				info("cpu_freq: UserSpace governor "
-				     "defined on cpu 0");
-			}
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: UserSpace governor defined on cpu 0");
+		}
+		if (strstr(value, "schedutil")) {
+			cpufreq[i].avail_governors |= GOV_SCHEDUTIL;
+			if (i == 0)
+				log_flag(CPU_FREQ, "cpu_freq: SchedUtil governor defined on cpu 0");
 		}
 		fclose(fp);
 		if (_cpu_freq_cpu_avail(i) == SLURM_ERROR)
 			continue;
-		if ((i == 0) && (debug_flags & DEBUG_FLAG_CPU_FREQ)) {
+		if ((i == 0) &&
+		    (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ)) {
 			for (j = 0; j < cpufreq[i].nfreq; j++) {
 				info("cpu_freq: frequency %u defined on cpu 0",
 				     cpufreq[i].avail_freq[j]);
@@ -384,17 +378,6 @@ cpu_freq_fini(void)
 	xfree(slurmd_spooldir);
 }
 
-/*
- * reset debug flag (slurmd)
- */
-extern void
-cpu_freq_reconfig(void)
-{
-	/* reset local static variables */
-	cpu_freq_govs = 0;
-	debug_flags = slurm_get_debug_flags();
-}
-
 /*
  * Send the cpu_frequency table info to slurmstepd
  */
@@ -445,11 +428,11 @@ rwfail:
 
 /*
  * Validate the cpus and select the frequency to set
- * Called from task cpuset code with task launch request containing
+ * Called from task/affinity code with task launch request containing
  *  a pointer to a hex map string of the cpus to be used by this step
  */
 extern void
-cpu_freq_cpuset_validate(stepd_step_rec_t *job)
+cpu_freq_cpuset_validate(stepd_step_rec_t *step)
 {
 	int cpuidx, cpu_num;
 	bitstr_t *cpus_to_set;
@@ -457,42 +440,49 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 	char *cpu_bind;
 	char *cpu_str;
 	char *savestr = NULL;
+	char cpu_bind_type_string[128];
 
 	if (set_batch_freq == -1) {
-		char *launch_params = slurm_get_launch_params();
-		if (xstrcasestr(launch_params, "batch_step_set_cpu_freq"))
+		if (xstrcasestr(slurm_conf.launch_params,
+				"batch_step_set_cpu_freq"))
 			set_batch_freq = 1;
 		else
 			set_batch_freq = 0;
-		xfree(launch_params);
 	}
 
-	if (((job->stepid == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
-	    (job->stepid == SLURM_EXTERN_CONT))
+	if (((step->step_id.step_id == SLURM_BATCH_SCRIPT) &&
+	     !set_batch_freq) ||
+	    (step->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    (step->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
-	debug_flags = slurm_get_debug_flags(); /* init for slurmstepd */
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
-		info("cpu_freq_cpuset_validate: request: min=(%12d  %8x) "
-		      "max=(%12d %8x) governor=%8x",
-		      job->cpu_freq_min, job->cpu_freq_min,
-		      job->cpu_freq_max, job->cpu_freq_max,
-		      job->cpu_freq_gov);
-		info("  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
-		     job->jobid, job->stepid, job->node_tasks,
-		     job->cpus_per_task, job->cpus);
-		info("  cpu_bind_type=%4x, cpu_bind map=%s",
-		     job->cpu_bind_type, job->cpu_bind);
-	}
+	slurm_sprint_cpu_bind_type(cpu_bind_type_string, step->cpu_bind_type);
+
+	log_flag(CPU_FREQ, "%s: request: min=(%12d  %8x) max=(%12d %8x) governor=%8x",
+		 __func__, step->cpu_freq_min, step->cpu_freq_min,
+		 step->cpu_freq_max, step->cpu_freq_max, step->cpu_freq_gov);
+	log_flag(CPU_FREQ, "  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
+		 step->step_id.job_id, step->step_id.step_id,
+		 step->node_tasks, step->cpus_per_task,
+		 step->cpus);
+	log_flag(CPU_FREQ, "  cpu_bind_type=%4x, cpu_bind map=%s",
+		 step->cpu_bind_type, step->cpu_bind);
 
 	if (!cpu_freq_count)
 		return;
 
-	if (job->cpu_bind == NULL) {
-		error("cpu_freq_cpuset_validate: cpu_bind string is null");
+	if (step->cpu_bind == NULL) {
+		/*
+		 * slurm_verify_cpu_bind will set cpu_bind to NULL for manual
+		 * binding that doesn't require an argument
+		 */
+		if ((step->cpu_bind_type != CPU_BIND_NONE) &&
+		    (step->cpu_bind_type != CPU_BIND_RANK) &&
+		    (step->cpu_bind_type != CPU_BIND_LDRANK))
+			error("cpu_freq_cpuset_validate: cpu_bind string is null");
 		return;
 	}
-	cpu_bind = xstrdup(job->cpu_bind);
+	cpu_bind = xstrdup(step->cpu_bind);
 
 	if ( (cpu_str = strtok_r(cpu_bind, ",", &savestr) ) == NULL) {
 		error("cpu_freq_cpuset_validate: cpu_bind string invalid");
@@ -506,13 +496,13 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 	do {
 		debug3("  cpu_str = %s", cpu_str);
 
-		if ((job->cpu_bind_type & CPU_BIND_MAP) == CPU_BIND_MAP) {
+		if ((step->cpu_bind_type & CPU_BIND_MAP) == CPU_BIND_MAP) {
 			cpu_num = atoi(cpu_str);
 			if (cpu_num >= cpu_freq_count) {
 				error("cpu_freq_cpuset_validate: invalid cpu "
 				      "number %d", cpu_num);
-				bit_free(cpu_map);
-				bit_free(cpus_to_set);
+				FREE_NULL_BITMAP(cpu_map);
+				FREE_NULL_BITMAP(cpus_to_set);
 				xfree(cpu_bind);
 				return;
 			}
@@ -521,8 +511,8 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 			if (bit_unfmt_hexmask(cpu_map, cpu_str) == -1) {
 				error("cpu_freq_cpuset_validate: invalid cpu "
 				      "mask %s", cpu_bind);
-				bit_free(cpu_map);
-				bit_free(cpus_to_set);
+				FREE_NULL_BITMAP(cpu_map);
+				FREE_NULL_BITMAP(cpus_to_set);
 				xfree(cpu_bind);
 				return;
 			}
@@ -532,13 +522,13 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
 
 	for (cpuidx = 0; cpuidx < cpu_freq_count; cpuidx++) {
 		if (bit_test(cpus_to_set, cpuidx)) {
-			_cpu_freq_setup_data(job, cpuidx);
+			_cpu_freq_setup_data(step, cpuidx);
 		}
 	}
-	cpu_freq_set(job);
+	cpu_freq_set(step);
 
-	bit_free(cpu_map);
-	bit_free(cpus_to_set);
+	FREE_NULL_BITMAP(cpu_map);
+	FREE_NULL_BITMAP(cpus_to_set);
 	xfree(cpu_bind);
 	return;
 }
@@ -549,7 +539,7 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *job)
  *  the list of cpus to be used by this step
  */
 extern void
-cpu_freq_cgroup_validate(stepd_step_rec_t *job, char *step_alloc_cores)
+cpu_freq_cgroup_validate(stepd_step_rec_t *step, char *step_alloc_cores)
 {
 	uint16_t start  = USHRT_MAX;
 	uint16_t end    = USHRT_MAX;
@@ -557,33 +547,31 @@ cpu_freq_cgroup_validate(stepd_step_rec_t *job, char *step_alloc_cores)
 	char *core_range;
 
 	if (set_batch_freq == -1) {
-		char *launch_params = slurm_get_launch_params();
-		if (xstrcasestr(launch_params, "batch_step_set_cpu_freq"))
+		if (xstrcasestr(slurm_conf.launch_params,
+				"batch_step_set_cpu_freq"))
 			set_batch_freq = 1;
 		else
 			set_batch_freq = 0;
-		xfree(launch_params);
 	}
 
-	if (((job->stepid == SLURM_BATCH_SCRIPT) && !set_batch_freq) ||
-	    (job->stepid == SLURM_EXTERN_CONT))
+	if (((step->step_id.step_id == SLURM_BATCH_SCRIPT) &&
+	     !set_batch_freq) ||
+	    (step->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    (step->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
-	debug_flags = slurm_get_debug_flags(); /* init for slurmstepd */
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
-		info("cpu_freq_cgroup_validate: request: min=(%12d  %8x) "
-				"max=(%12d %8x) governor=%8x",
-		       job->cpu_freq_min, job->cpu_freq_min,
-		       job->cpu_freq_max, job->cpu_freq_max,
-		       job->cpu_freq_gov);
-		info("  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
-		       job->jobid,job->stepid,job->node_tasks,
-		       job->cpus_per_task,job->cpus);
-		info("  cpu_bind_type=%4x, cpu_bind map=%s",
-		       job->cpu_bind_type, job->cpu_bind);
-		info("  step logical cores = %s, step physical cores = %s",
-		       job->step_alloc_cores, step_alloc_cores);
-	}
+	log_flag(CPU_FREQ, "%s: request: min=(%12d  %8x) max=(%12d %8x) governor=%8x",
+		 __func__, step->cpu_freq_min, step->cpu_freq_min,
+		 step->cpu_freq_max, step->cpu_freq_max, step->cpu_freq_gov);
+	log_flag(CPU_FREQ, "  jobid=%u, stepid=%u, tasks=%u cpu/task=%u, cpus=%u",
+		 step->step_id.job_id, step->step_id.step_id,
+		 step->node_tasks, step->cpus_per_task,
+		 step->cpus);
+	log_flag(CPU_FREQ, "  cpu_bind_type=%4x, cpu_bind map=%s",
+		 step->cpu_bind_type, step->cpu_bind);
+	log_flag(CPU_FREQ, "  step logical cores = %s, step physical cores = %s",
+		 step->step_alloc_cores, step_alloc_cores);
+
 	if (!cpu_freq_count)
 		return;
 
@@ -596,9 +584,9 @@ cpu_freq_cgroup_validate(stepd_step_rec_t *job, char *step_alloc_cores)
 			  cpuidx, cpu_freq_count);
 		    return;
 		}
-		_cpu_freq_setup_data(job, cpuidx);
+		_cpu_freq_setup_data(step, cpuidx);
 	}
-	cpu_freq_set(job);
+	cpu_freq_set(step);
 	return;
 }
 
@@ -715,16 +703,26 @@ _cpu_freq_get_cur_gov(int cpuidx)
  * set cpu governor
  */
 static int
-_cpu_freq_set_gov(stepd_step_rec_t *job, int cpuidx, char* gov )
+_cpu_freq_set_gov(stepd_step_rec_t *step, int cpuidx, char *gov)
 {
 	char path[PATH_MAX];
 	FILE *fp;
 	int fd, rc;
+	uint32_t jobid;
+
+#ifdef HAVE_NATIVE_CRAY
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
+	else
+		jobid = step->step_id.job_id;
+#else
+	jobid = step->step_id.job_id;
+#endif
 
 	rc = SLURM_SUCCESS;
 	snprintf(path, sizeof(path), PATH_TO_CPU
 		 "cpu%u/cpufreq/scaling_governor", cpuidx);
-	fd = _set_cpu_owner_lock(cpuidx, job->jobid);
+	fd = _set_cpu_owner_lock(cpuidx, jobid);
 	if ((fp = fopen(path, "w"))) {
 		fputs(gov, fp);
 		fputc('\n', fp);
@@ -768,7 +766,7 @@ _cpu_freq_get_scaling_freq(int cpuidx, char* option)
 }
 
 /*
- * test for existance of cpufreq file
+ * test for existence of cpufreq file
  *
  * Return: true if file found
  */
@@ -792,18 +790,28 @@ _cpu_freq_test_scaling_freq(int cpuidx, char *option)
  *
  */
 static int
-_cpu_freq_set_scaling_freq(stepd_step_rec_t *job, int cpx, uint32_t freq,
+_cpu_freq_set_scaling_freq(stepd_step_rec_t *step, int cpx, uint32_t freq,
 		char* option)
 {
 	char path[PATH_MAX];
 	FILE *fp;
 	int fd, rc;
 	uint32_t newfreq;
+	uint32_t jobid;
+
+#ifdef HAVE_NATIVE_CRAY
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
+	else
+		jobid = step->step_id.job_id;
+#else
+	jobid = step->step_id.job_id;
+#endif
 
 	rc = SLURM_SUCCESS;
 	snprintf(path, sizeof(path), PATH_TO_CPU
 		 "cpu%u/cpufreq/%s", cpx, option);
-	fd = _set_cpu_owner_lock(cpx, job->jobid);
+	fd = _set_cpu_owner_lock(cpx, jobid);
 	if ((fp = fopen(path, "w"))) {
 		fprintf(fp, "%u\n", freq);
 		fclose(fp);
@@ -815,7 +823,7 @@ _cpu_freq_set_scaling_freq(stepd_step_rec_t *job, int cpx, uint32_t freq,
 		(void) fd_release_lock(fd);
 		(void) close(fd);
 	}
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 		newfreq = _cpu_freq_get_scaling_freq(cpx, option);
 		if (newfreq != freq) {
 			error("Failed to set freq_scaling %s to %u (org=%u)",
@@ -898,7 +906,7 @@ _cpu_freq_govspec_string(uint32_t cpu_freq, int cpuidx)
 
 	if ((cpu_freq & CPU_FREQ_RANGE_FLAG) == 0)
 		return SLURM_ERROR;
-		
+
 	switch(cpu_freq)
 	{
 	case CPU_FREQ_CONSERVATIVE:
@@ -921,6 +929,10 @@ _cpu_freq_govspec_string(uint32_t cpu_freq, int cpuidx)
 		if (cpufreq[cpuidx].avail_governors & GOV_USERSPACE)
 			strcpy(cpufreq[cpuidx].new_governor, "userspace");
 		return SLURM_SUCCESS;
+	case CPU_FREQ_SCHEDUTIL:
+		if (cpufreq[cpuidx].avail_governors & GOV_SCHEDUTIL)
+			strcpy(cpufreq[cpuidx].new_governor, "schedutil");
+		return SLURM_SUCCESS;
 	default:
 		return SLURM_ERROR;
 	}
@@ -942,27 +954,27 @@ _cpu_freq_freqspec_num(uint32_t cpu_freq, int cpuidx)
 		{
 		case CPU_FREQ_LOW :
 			return cpufreq[cpuidx].avail_freq[0];
-	
+
 		case CPU_FREQ_MEDIUM :
 			if (cpufreq[cpuidx].nfreq == 1)
 				return cpufreq[cpuidx].avail_freq[0];
 			fx = (cpufreq[cpuidx].nfreq - 1) / 2;
 			return cpufreq[cpuidx].avail_freq[fx];
-	
+
 		case CPU_FREQ_HIGHM1 :
 			if (cpufreq[cpuidx].nfreq == 1)
 				return cpufreq[cpuidx].avail_freq[0];
 			fx = cpufreq[cpuidx].nfreq - 2;
 			return cpufreq[cpuidx].avail_freq[fx];
-	
+
 		case CPU_FREQ_HIGH :
 			fx = cpufreq[cpuidx].nfreq - 1;
 			return cpufreq[cpuidx].avail_freq[fx];
-		
+
 		default:
 			return NO_VAL;
 		}
-	}		
+	}
 
 	/* check for request above or below available values */
 	if (cpu_freq < cpufreq[cpuidx].avail_freq[0]) {
@@ -1019,18 +1031,18 @@ _cpu_freq_init_data(int cpx)
  * Or min/max governor base on --cpu-freq parameter
  */
 static void
-_cpu_freq_setup_data(stepd_step_rec_t *job, int cpx)
+_cpu_freq_setup_data(stepd_step_rec_t *step, int cpx)
 {
 	uint32_t freq;
 
-	if (   (job->cpu_freq_min == NO_VAL || job->cpu_freq_min==0)
-	    && (job->cpu_freq_max == NO_VAL || job->cpu_freq_max==0)
-	    && (job->cpu_freq_gov == NO_VAL || job->cpu_freq_gov==0)) {
+	if (   (step->cpu_freq_min == NO_VAL || step->cpu_freq_min==0)
+	    && (step->cpu_freq_max == NO_VAL || step->cpu_freq_max==0)
+	    && (step->cpu_freq_gov == NO_VAL || step->cpu_freq_gov==0)) {
 		/* If no --cpu-freq, use default governor from conf file.  */
-		slurm_ctl_conf_t *conf = slurm_conf_lock();
-		job->cpu_freq_gov = conf->cpu_freq_def;
+		slurm_conf_t *conf = slurm_conf_lock();
+		step->cpu_freq_gov = conf->cpu_freq_def;
 		slurm_conf_unlock();
-		if (job->cpu_freq_gov == NO_VAL)
+		if (step->cpu_freq_gov == NO_VAL)
 			return;
 	}
 
@@ -1038,35 +1050,35 @@ _cpu_freq_setup_data(stepd_step_rec_t *job, int cpx)
 	if (_cpu_freq_current_state(cpx) == SLURM_ERROR)
 		return;
 
-	if (job->cpu_freq_min == NO_VAL &&
-	    job->cpu_freq_max != NO_VAL &&
-	    job->cpu_freq_gov == NO_VAL) {
+	if (step->cpu_freq_min == NO_VAL &&
+	    step->cpu_freq_max != NO_VAL &&
+	    step->cpu_freq_gov == NO_VAL) {
 		/* Pre version 15.08 behavior */
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_max, cpx);
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_max, cpx);
 		cpufreq[cpx].new_frequency = freq;
 		goto newfreq;
 	}
-	if (job->cpu_freq_gov == CPU_FREQ_USERSPACE) {
-		_cpu_freq_govspec_string(job->cpu_freq_gov, cpx);
-		if (job->cpu_freq_max == NO_VAL) {
+	if (step->cpu_freq_gov == CPU_FREQ_USERSPACE) {
+		_cpu_freq_govspec_string(step->cpu_freq_gov, cpx);
+		if (step->cpu_freq_max == NO_VAL) {
 			return; /* pre version 15.08 behavior. */
 		}
 		/* Power capping */
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_max, cpx);
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_max, cpx);
 		cpufreq[cpx].new_frequency = freq;
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_min, cpx);
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_min, cpx);
 		cpufreq[cpx].new_min_freq = freq;
 		goto newfreq;
 	}
-	if (job->cpu_freq_min != NO_VAL && job->cpu_freq_max != NO_VAL) {
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_min, cpx);
+	if (step->cpu_freq_min != NO_VAL && step->cpu_freq_max != NO_VAL) {
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_min, cpx);
 		cpufreq[cpx].new_min_freq = freq;
-		freq = _cpu_freq_freqspec_num(job->cpu_freq_max, cpx);
+		freq = _cpu_freq_freqspec_num(step->cpu_freq_max, cpx);
 		cpufreq[cpx].new_max_freq = freq;
 	}
 
-	if (job->cpu_freq_gov != NO_VAL) {
-		_cpu_freq_govspec_string(job->cpu_freq_gov, cpx);
+	if (step->cpu_freq_gov != NO_VAL) {
+		_cpu_freq_govspec_string(step->cpu_freq_gov, cpx);
 	}
 newfreq:
 	/* Make sure a 'new' frequency is within scaling min/max */
@@ -1102,6 +1114,8 @@ _cpu_freq_check_gov(const char* arg, uint32_t illegal)
 		rc = CPU_FREQ_USERSPACE;
 	} else if (xstrncasecmp(arg, "onde", 4) == 0) {
 		rc = CPU_FREQ_ONDEMAND;
+	} else if (xstrncasecmp(arg, "sche", 4) == 0) {
+		rc = CPU_FREQ_SCHEDUTIL;
 	}
 	rc &= (~illegal);
 	if (rc == 0)
@@ -1145,7 +1159,7 @@ _cpu_freq_check_freq(const char* arg)
  * set cpu frequency if possible for each cpu of the job step
  */
 extern void
-cpu_freq_set(stepd_step_rec_t *job)
+cpu_freq_set(stepd_step_rec_t *step)
 {
 	char freq_detail[100];
 	uint32_t freq;
@@ -1160,14 +1174,10 @@ cpu_freq_set(stepd_step_rec_t *job)
 	            && cpufreq[i].new_max_freq == NO_VAL
 		    && cpufreq[i].new_governor[0] == '\0')
 			continue; /* Nothing to set on this CPU */
-		if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
-			info("cpu_freq: current_state cpu=%d org_min=%u "
-			     "org_freq=%u org_max=%u org_gpv=%s", i,
-			     cpufreq[i].org_min_freq,
-			     cpufreq[i].org_frequency,
-			     cpufreq[i].org_max_freq,
-			     cpufreq[i].org_governor);
-		}
+
+		log_flag(CPU_FREQ, "cpu_freq: current_state cpu=%d org_min=%u org_freq=%u org_max=%u org_gpv=%s",
+			 i, cpufreq[i].org_min_freq, cpufreq[i].org_frequency,
+			 cpufreq[i].org_max_freq, cpufreq[i].org_governor);
 
 		/* Max must be set before min, per
 		 * www.kernel.org/doc/Documentation/cpu-freq/user-guide.txt
@@ -1178,22 +1188,22 @@ cpu_freq_set(stepd_step_rec_t *job)
 				/* The current frequency is > requested max,
 				 * Set it so it is in range
 				 * have to go to UserSpace to do it. */
-				rc = _cpu_freq_set_gov(job, i, "userspace");
+				rc = _cpu_freq_set_gov(step, i, "userspace");
 				if (rc == SLURM_ERROR)
 					return;
-				rc = _cpu_freq_set_scaling_freq(job, i, freq,
+				rc = _cpu_freq_set_scaling_freq(step, i, freq,
 						         "scaling_setspeed");
 				if (rc == SLURM_ERROR)
 					continue;
 				if (cpufreq[i].new_governor[0] == '\0') {
 					/* Not requesting new gov, so restore */
-					rc = _cpu_freq_set_gov(job, i,
+					rc = _cpu_freq_set_gov(step, i,
 						cpufreq[i].org_governor);
 					if (rc == SLURM_ERROR)
 						continue;
 				}
 			}
-			rc = _cpu_freq_set_scaling_freq(job, i, freq,
+			rc = _cpu_freq_set_scaling_freq(step, i, freq,
 							"scaling_max_freq");
 			if (rc == SLURM_ERROR)
 				continue;
@@ -1204,44 +1214,44 @@ cpu_freq_set(stepd_step_rec_t *job)
 				/* The current frequency is < requested min,
 				 * Set it so it is in range
 				 * have to go to UserSpace to do it. */
-				rc = _cpu_freq_set_gov(job, i, "userspace");
+				rc = _cpu_freq_set_gov(step, i, "userspace");
 				if (rc == SLURM_ERROR)
 					continue;
-				rc = _cpu_freq_set_scaling_freq(job, i, freq,
+				rc = _cpu_freq_set_scaling_freq(step, i, freq,
 						         "scaling_setspeed");
 				if (rc == SLURM_ERROR)
 					continue;
 				if (cpufreq[i].new_governor[0] == '\0') {
 					/* Not requesting new gov, so restore */
-					rc= _cpu_freq_set_gov(job, i,
+					rc= _cpu_freq_set_gov(step, i,
 						cpufreq[i].org_governor);
 					if (rc == SLURM_ERROR)
 						continue;
 				}
 			}
-			rc= _cpu_freq_set_scaling_freq(job, i, freq,
+			rc= _cpu_freq_set_scaling_freq(step, i, freq,
 						       "scaling_min_freq");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_frequency != NO_VAL) {
 			if (xstrcmp(cpufreq[i].org_governor,"userspace")) {
-				rc = _cpu_freq_set_gov(job, i, "userspace");
+				rc = _cpu_freq_set_gov(step, i, "userspace");
 				if (rc == SLURM_ERROR)
 					continue;
 			}
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].new_frequency,
 					"scaling_setspeed");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_governor[0] != '\0') {
-			rc = _cpu_freq_set_gov(job, i, cpufreq[i].new_governor);
+			rc = _cpu_freq_set_gov(step, i, cpufreq[i].new_governor);
 			if (rc == SLURM_ERROR)
 				continue;
 		}
-		if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+		if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 			cpu_freq_debug(NULL, NULL,
 					freq_detail, sizeof(freq_detail),
 					NO_VAL, cpufreq[i].new_min_freq,
@@ -1263,7 +1273,7 @@ cpu_freq_set(stepd_step_rec_t *job)
  * default frequency and governor type
  */
 extern void
-cpu_freq_reset(stepd_step_rec_t *job)
+cpu_freq_reset(stepd_step_rec_t *step)
 {
 	int i, rc, fd;
 	char freq_detail[100];
@@ -1273,12 +1283,12 @@ cpu_freq_reset(stepd_step_rec_t *job)
 		return;
 
 #ifdef HAVE_NATIVE_CRAY
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
 	else
-		jobid = job->jobid;
+		jobid = step->step_id.job_id;
 #else
-	jobid = job->jobid;
+	jobid = step->step_id.job_id;
 #endif
 
 	for (i = 0; i < cpu_freq_count; i++) {
@@ -1293,10 +1303,10 @@ cpu_freq_reset(stepd_step_rec_t *job)
 			continue;
 
 		if (cpufreq[i].new_frequency != NO_VAL) {
-			rc = _cpu_freq_set_gov(job, i, "userspace");
+			rc = _cpu_freq_set_gov(step, i, "userspace");
 			if (rc == SLURM_ERROR)
 				continue;
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].org_frequency,
 					"scaling_setspeed");
 			if (rc == SLURM_ERROR)
@@ -1307,26 +1317,27 @@ cpu_freq_reset(stepd_step_rec_t *job)
 		 * www.kernel.org/doc/Documentation/cpu-freq/user-guide.txt
 		 */
 		if (cpufreq[i].new_max_freq != NO_VAL) {
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].org_max_freq,
 					"scaling_max_freq");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_min_freq != NO_VAL) {
-			rc = _cpu_freq_set_scaling_freq(job, i,
+			rc = _cpu_freq_set_scaling_freq(step, i,
 					cpufreq[i].org_min_freq,
 					"scaling_min_freq");
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 		if (cpufreq[i].new_governor[0] != '\0') {
-			rc = _cpu_freq_set_gov(job, i, cpufreq[i].org_governor);
+			rc = _cpu_freq_set_gov(
+				step, i, cpufreq[i].org_governor);
 			if (rc == SLURM_ERROR)
 				continue;
 		}
 
-		if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+		if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 			cpu_freq_debug(NULL, NULL,
 					freq_detail, sizeof(freq_detail),
 					NO_VAL, cpufreq[i].org_min_freq,
@@ -1365,6 +1376,8 @@ cpu_freq_to_string(char *buf, int buf_size, uint32_t cpu_freq)
 		snprintf(buf, buf_size, "UserSpace");
 	else if (cpu_freq == CPU_FREQ_ONDEMAND)
 		snprintf(buf, buf_size, "OnDemand");
+	else if (cpu_freq == CPU_FREQ_SCHEDUTIL)
+		snprintf(buf, buf_size, "SchedUtil");
 	else if (cpu_freq & CPU_FREQ_RANGE_FLAG)
 		snprintf(buf, buf_size, "Unknown");
 	else if (fuzzy_equal(cpu_freq, NO_VAL)) {
@@ -1454,46 +1467,31 @@ extern void
 cpu_freq_govlist_to_string(char* buf, uint16_t bufsz, uint32_t govs)
 {
 	char *list = NULL;
+	char *sep = "", *pos = NULL;
 
 	if ((govs & CPU_FREQ_CONSERVATIVE) == CPU_FREQ_CONSERVATIVE) {
-		if (list == NULL)
-			list = xstrdup("Conservative");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"Conservative");
-		}
-	}
-	if ((govs & CPU_FREQ_PERFORMANCE) == CPU_FREQ_PERFORMANCE) {
-		if (list == NULL)
-			list = xstrdup("Performance");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"Performance");
-		}
-	}
-	if ((govs & CPU_FREQ_POWERSAVE) == CPU_FREQ_POWERSAVE) {
-		if (list == NULL)
-			list = xstrdup("PowerSave");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"PowerSave");
-		}
+		xstrfmtcatat(list, &pos, "%s%s", sep, "Conservative");
+		sep = ",";
 	}
 	if ((govs & CPU_FREQ_ONDEMAND) == CPU_FREQ_ONDEMAND) {
-		if (list == NULL)
-			list = xstrdup("OnDemand");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"OnDemand");
-		}
+		xstrfmtcatat(list, &pos, "%s%s", sep, "OnDemand");
+		sep = ",";
+	}
+	if ((govs & CPU_FREQ_PERFORMANCE) == CPU_FREQ_PERFORMANCE) {
+		xstrfmtcatat(list, &pos, "%s%s", sep, "Performance");
+		sep = ",";
+	}
+	if ((govs & CPU_FREQ_POWERSAVE) == CPU_FREQ_POWERSAVE) {
+		xstrfmtcatat(list, &pos, "%s%s", sep, "PowerSave");
+		sep = ",";
+	}
+	if ((govs & CPU_FREQ_SCHEDUTIL) == CPU_FREQ_SCHEDUTIL) {
+		xstrfmtcatat(list, &pos, "%s%s", sep, "SchedUtil");
+		sep = ",";
 	}
 	if ((govs & CPU_FREQ_USERSPACE) == CPU_FREQ_USERSPACE) {
-		if (list == NULL)
-			list = xstrdup("UserSpace");
-		else {
-			xstrcatchar(list,',');
-			xstrcat(list,"UserSpace");
-		}
+		xstrfmtcatat(list, &pos, "%s%s", sep, "UserSpace");
+		sep = ",";
 	}
 
 	if (list) {
@@ -1601,10 +1599,6 @@ cpu_freq_verify_cmdline(const char *arg,
 	uint32_t frequency;
 	int rc = 0;
 
-	if (cpu_freq_govs == 0)
-		cpu_freq_govs = slurm_get_cpu_freq_govs();
-
-
 	if (arg == NULL || cpu_freq_min == NULL || cpu_freq_max == NULL
 			|| cpu_freq_gov == NULL) {
 		return -1;
@@ -1682,7 +1676,7 @@ cpu_freq_verify_cmdline(const char *arg,
 
 clean:
 	if (*cpu_freq_gov != NO_VAL) {
-		if (((*cpu_freq_gov & cpu_freq_govs)
+		if (((*cpu_freq_gov & slurm_conf.cpu_freq_govs)
 		    & ~CPU_FREQ_RANGE_FLAG) == 0) {
 			error("governor of %s is not allowed in slurm.conf",
 			      arg);
@@ -1690,7 +1684,7 @@ clean:
 			rc = -1;
 		}
 	}
-	if (debug_flags & DEBUG_FLAG_CPU_FREQ) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_CPU_FREQ) {
 		cpu_freq_debug("command", "NO_VAL", NULL, 0,
 			       *cpu_freq_gov, *cpu_freq_min,
 			       *cpu_freq_max, NO_VAL);
