@@ -1,7 +1,6 @@
 /*****************************************************************************\
  *  as_mysql_cluster.c - functions dealing with clusters.
  *****************************************************************************
- *
  *  Copyright (C) 2004-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2011 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -43,7 +42,9 @@
 #include "as_mysql_federation.h"
 #include "as_mysql_usage.h"
 #include "as_mysql_wckey.h"
-#include "src/common/node_select.h"
+
+#include "src/interfaces/select.h"
+#include "src/common/slurm_time.h"
 
 extern int as_mysql_get_fed_cluster_id(mysql_conn_t *mysql_conn,
 				       const char *cluster,
@@ -70,8 +71,7 @@ extern int as_mysql_get_fed_cluster_id(mysql_conn_t *mysql_conn,
 			  "FROM %s "
 			  "WHERE deleted=0 AND name='%s' AND federation='%s';",
 		   cluster_table, cluster, federation);
-	if (debug_flags & DEBUG_FLAG_FEDR)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(FEDR, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		error("no result given for %s", query);
@@ -80,10 +80,8 @@ extern int as_mysql_get_fed_cluster_id(mysql_conn_t *mysql_conn,
 	xfree(query);
 	while ((row = mysql_fetch_row(result))) {
 		int tmp_id = slurm_atoul(row[1]);
-		if (debug_flags & DEBUG_FLAG_FEDR)
-			info("cluster '%s' already part of federation '%s', "
-			     "using existing id %d", cluster, federation,
-			     tmp_id);
+		log_flag(FEDR, "cluster '%s' already part of federation '%s', using existing id %d",
+			 cluster, federation, tmp_id);
 		mysql_free_result(result);
 		*ret_id = tmp_id;
 		return SLURM_SUCCESS;
@@ -96,8 +94,7 @@ extern int as_mysql_get_fed_cluster_id(mysql_conn_t *mysql_conn,
 		   	  "WHERE name!='%s' AND federation='%s' "
 			  "AND fed_id > %d AND deleted=0 ORDER BY fed_id;",
 			  cluster_table, cluster, federation, last_id);
-	if (debug_flags & DEBUG_FLAG_FEDR)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(FEDR, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		error("no result given for %s", query);
@@ -337,8 +334,7 @@ extern int as_mysql_add_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 			   now, object->classification,
 			   (object->fed.name) ? object->fed.name : "",
 			   fed_id, fed_state, (features) ? features : "");
-		if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_ASSOC, mysql_conn->conn, "query\n%s", query);
 		rc = mysql_db_query(mysql_conn, query);
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
@@ -374,8 +370,7 @@ extern int as_mysql_add_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 				   extra);
 			xfree(cols);
 			xfree(vals);
-			if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+			DB_DEBUG(DB_ASSOC, mysql_conn->conn, "query\n%s", query);
 
 			rc = mysql_db_query(mysql_conn, query);
 			xfree(query);
@@ -423,7 +418,7 @@ extern int as_mysql_add_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 
 			added++;
 			/* add it to the list and sort */
-			slurm_mutex_lock(&as_mysql_cluster_list_lock);
+			slurm_rwlock_wrlock(&as_mysql_cluster_list_lock);
 			check_itr = list_iterator_create(as_mysql_cluster_list);
 			while ((tmp_name = list_next(check_itr))) {
 				if (!xstrcmp(tmp_name, object->name))
@@ -439,7 +434,7 @@ extern int as_mysql_add_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 				error("Cluster %s(%s) appears to already be in "
 				      "our cache list, not adding.", tmp_name,
 				      object->name);
-			slurm_mutex_unlock(&as_mysql_cluster_list_lock);
+			slurm_rwlock_unlock(&as_mysql_cluster_list_lock);
 		}
 
 		if (!external_cluster) {
@@ -455,7 +450,12 @@ extern int as_mysql_add_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 			assoc->user = xstrdup("root");
 			assoc->acct = xstrdup("root");
 			assoc->is_def = 1;
-
+			/*
+			 * If the cluster is registering then don't add to the
+			 * update_list.
+			 */
+			if (object->flags & CLUSTER_FLAG_REGISTER)
+				assoc->flags |= ASSOC_FLAG_NO_UPDATE;
 			if (as_mysql_add_assocs(mysql_conn, uid, assoc_list)
 			    == SLURM_ERROR) {
 				error("Problem adding root user association");
@@ -542,6 +542,7 @@ extern List as_mysql_modify_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 		xstrfmtcat(vals, ", control_host='%s'", cluster->control_host);
 		set++;
 		clust_reg = true;
+		fed_update = true;
 	}
 
 	if (cluster->control_port) {
@@ -549,12 +550,14 @@ extern List as_mysql_modify_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 			   cluster->control_port, cluster->control_port);
 		set++;
 		clust_reg = true;
+		fed_update = true;
 	}
 
 	if (cluster->rpc_version) {
 		xstrfmtcat(vals, ", rpc_version=%u", cluster->rpc_version);
 		set++;
 		clust_reg = true;
+		fed_update = true;
 	}
 
 	if (cluster->dimensions) {
@@ -604,8 +607,7 @@ extern List as_mysql_modify_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 	xstrfmtcat(query, "select name, control_port, federation, features from %s%s;",
 		   cluster_table, extra);
 
-	if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_ASSOC, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(
 		      mysql_conn, query, 0))) {
 		xfree(query);
@@ -715,9 +717,8 @@ extern List as_mysql_modify_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 
 	if (!list_count(ret_list)) {
 		errno = SLURM_NO_CHANGE_IN_DATA;
-		if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-			DB_DEBUG(mysql_conn->conn,
-				 "didn't effect anything\n%s", query);
+		DB_DEBUG(DB_ASSOC, mysql_conn->conn,
+		         "didn't affect anything\n%s", query);
 		xfree(name_char);
 		xfree(vals);
 		xfree(query);
@@ -789,9 +790,8 @@ extern List as_mysql_remove_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 	if (!mysql_num_rows(result)) {
 		mysql_free_result(result);
 		errno = SLURM_NO_CHANGE_IN_DATA;
-		if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-			DB_DEBUG(mysql_conn->conn,
-				 "didn't effect anything\n%s", query);
+		DB_DEBUG(DB_ASSOC, mysql_conn->conn,
+		         "didn't affect anything\n%s", query);
 		xfree(query);
 		return ret_list;
 	}
@@ -829,7 +829,8 @@ extern List as_mysql_remove_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 			   object, cluster_month_table, now);
 		rc = remove_common(mysql_conn, DBD_REMOVE_CLUSTERS, now,
 				   user_name, cluster_table, name_char,
-				   assoc_char, object, ret_list, &jobs_running);
+				   assoc_char, object, ret_list, &jobs_running,
+				   NULL);
 		xfree(object);
 		if (rc != SLURM_SUCCESS)
 			break;
@@ -845,8 +846,7 @@ extern List as_mysql_remove_clusters(mysql_conn_t *mysql_conn, uint32_t uid,
 		return NULL;
 	}
 	if (!jobs_running) {
-		if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_ASSOC, mysql_conn->conn, "query\n%s", query);
 		rc = mysql_db_query(mysql_conn, query);
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
@@ -966,8 +966,7 @@ empty:
 	xfree(tmp);
 	xfree(extra);
 
-	if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_ASSOC, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(
 		      mysql_conn, query, 0))) {
 		xfree(query);
@@ -1021,8 +1020,7 @@ empty:
 			"select tres, cluster_nodes from "
 			"\"%s_%s\" where time_end=0 and node_name='' limit 1",
 			cluster->name, event_table);
-		if (debug_flags & DEBUG_FLAG_DB_TRES)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_TRES, mysql_conn->conn, "query\n%s", query);
 		if (!(result2 = mysql_db_query_ret(mysql_conn, query, 0))) {
 			xfree(query);
 			continue;
@@ -1106,10 +1104,10 @@ extern List as_mysql_get_cluster_events(mysql_conn_t *mysql_conn, uint32_t uid,
 	int i=0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
-	uint16_t private_data = 0;
 	time_t now = time(NULL);
-	List use_cluster_list = as_mysql_cluster_list;
+	List use_cluster_list = NULL;
 	slurmdb_user_rec_t user;
+	bool locked = false;
 
 	/* if this changes you will need to edit the corresponding enum */
 	char *event_req_inx[] = {
@@ -1141,9 +1139,7 @@ extern List as_mysql_get_cluster_events(mysql_conn_t *mysql_conn, uint32_t uid,
 	memset(&user, 0, sizeof(slurmdb_user_rec_t));
 	user.uid = uid;
 
-	private_data = slurm_get_private_data();
-
-	if (private_data & PRIVATE_DATA_EVENTS) {
+	if (slurm_conf.private_data & PRIVATE_DATA_EVENTS) {
 		if (!is_user_min_admin_level(
 			      mysql_conn, uid, SLURMDB_ADMIN_OPERATOR)) {
 			error("UID %u tried to access events, only administrators can look at events",
@@ -1162,12 +1158,17 @@ extern List as_mysql_get_cluster_events(mysql_conn_t *mysql_conn, uint32_t uid,
 		else
 			xstrcat(extra, " where (");
 
+		xstrfmtcat(extra,
+			   "(CONVERT(SUBSTRING_INDEX(tres,'%d=',-1),"
+			   "UNSIGNED INTEGER)",
+			   TRES_CPU);
+
 		if (event_cond->cpus_max) {
-			xstrfmtcat(extra, "count between %u and %u)",
+			xstrfmtcat(extra, " between %u and %u))",
 				   event_cond->cpus_min, event_cond->cpus_max);
 
 		} else {
-			xstrfmtcat(extra, "count='%u')",
+			xstrfmtcat(extra, "='%u'))",
 				   event_cond->cpus_min);
 
 		}
@@ -1238,10 +1239,24 @@ extern List as_mysql_get_cluster_events(mysql_conn_t *mysql_conn, uint32_t uid,
 		else
 			xstrcat(extra, " where (");
 
-		xstrfmtcat(extra,
-			   "(time_start < %ld) "
-			   "&& (time_end >= %ld || time_end = 0))",
-			   event_cond->period_end, event_cond->period_start);
+		if (event_cond->cond_flags & SLURMDB_EVENT_COND_OPEN)
+			xstrfmtcat(extra,
+				   "(time_start >= %ld) && (time_end = 0))",
+				   event_cond->period_start);
+		else
+			xstrfmtcat(extra,
+				   "(time_start < %ld) "
+				   "&& (time_end >= %ld || time_end = 0))",
+				   event_cond->period_end,
+				   event_cond->period_start);
+
+	} else if (event_cond->cond_flags & SLURMDB_EVENT_COND_OPEN) {
+		if (extra)
+			xstrcat(extra, " && (");
+		else
+			xstrcat(extra, " where (");
+
+		xstrfmtcat(extra, "time_end = 0)");
 	}
 
 	if (event_cond->reason_list
@@ -1304,17 +1319,27 @@ extern List as_mysql_get_cluster_events(mysql_conn_t *mysql_conn, uint32_t uid,
 		xstrcat(extra, ")");
 	}
 
-	if (event_cond->cluster_list && list_count(event_cond->cluster_list))
-		use_cluster_list = event_cond->cluster_list;
 empty:
 	xfree(tmp);
 	xstrfmtcat(tmp, "%s", event_req_inx[0]);
 	for(i=1; i<EVENT_REQ_COUNT; i++) {
-		xstrfmtcat(tmp, ", %s", event_req_inx[i]);
+		bool include = true;
+		if (event_cond && event_cond->format_list)
+			include = list_find_first(event_cond->format_list,
+						  slurm_find_char_in_list,
+						  event_req_inx[i]);
+		xstrfmtcat(tmp, ", %s%s",
+			   include ? "" : "'' as ", event_req_inx[i]);
 	}
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_lock(&as_mysql_cluster_list_lock);
+	if (event_cond && event_cond->cluster_list &&
+	    list_count(event_cond->cluster_list)) {
+		use_cluster_list = event_cond->cluster_list;
+	} else {
+		slurm_rwlock_rdlock(&as_mysql_cluster_list_lock);
+		use_cluster_list = list_shallow_copy(as_mysql_cluster_list);
+		locked = true;
+	}
 
 	ret_list = list_create(slurmdb_destroy_event_rec);
 
@@ -1325,8 +1350,7 @@ empty:
 		if (extra)
 			xstrfmtcat(query, " %s", extra);
 
-		if (debug_flags & DEBUG_FLAG_DB_ASSOC)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_ASSOC, mysql_conn->conn, "query\n%s", query);
 		if (!(result = mysql_db_query_ret(
 			      mysql_conn, query, 0))) {
 			xfree(query);
@@ -1375,8 +1399,10 @@ empty:
 	xfree(tmp);
 	xfree(extra);
 
-	if (use_cluster_list == as_mysql_cluster_list)
-		slurm_mutex_unlock(&as_mysql_cluster_list_lock);
+	if (locked) {
+		FREE_NULL_LIST(use_cluster_list);
+		slurm_rwlock_unlock(&as_mysql_cluster_list_lock);
+	}
 
 	return ret_list;
 }
@@ -1433,12 +1459,10 @@ extern int as_mysql_node_down(mysql_conn_t *mysql_conn,
 	row = mysql_fetch_row(result);
 	if (row && (node_ptr->node_state == slurm_atoul(row[0])) &&
 	    !xstrcasecmp(my_reason, row[1])) {
-		if (debug_flags & DEBUG_FLAG_DB_EVENT)
-			DB_DEBUG(mysql_conn->conn,
-				 "no change to %s(%s) needed %u == %s and %s == %s",
-				 node_ptr->name, mysql_conn->cluster_name,
-				 node_ptr->node_state, row[0],
-				 my_reason, row[1]);
+		DB_DEBUG(DB_EVENT, mysql_conn->conn,
+		         "no change to %s(%s) needed %u == %s and %s == %s",
+		         node_ptr->name, mysql_conn->cluster_name,
+		         node_ptr->node_state, row[0], my_reason, row[1]);
 		mysql_free_result(result);
 		return SLURM_SUCCESS;
 	}
@@ -1459,8 +1483,7 @@ extern int as_mysql_node_down(mysql_conn_t *mysql_conn,
 			"time_start=%ld and node_name='%s';",
 			mysql_conn->cluster_name, event_table,
 			my_reason, event_time, node_ptr->name);
-		if (debug_flags & DEBUG_FLAG_DB_EVENT)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_EVENT, mysql_conn->conn, "query\n%s", query);
 		rc = mysql_db_query(mysql_conn, query);
 		xfree(query);
 
@@ -1470,27 +1493,33 @@ extern int as_mysql_node_down(mysql_conn_t *mysql_conn,
 
 	mysql_free_result(result);
 
-	if (debug_flags & DEBUG_FLAG_DB_EVENT)
-		DB_DEBUG(mysql_conn->conn,
-			 "inserting %s(%s) with tres of '%s'",
-			 node_ptr->name, mysql_conn->cluster_name,
-			 node_ptr->tres_str);
+	DB_DEBUG(DB_EVENT, mysql_conn->conn,
+	         "inserting %s(%s) with tres of '%s'",
+		 node_ptr->name, mysql_conn->cluster_name, node_ptr->tres_str);
 
 	query = xstrdup_printf(
 		"update \"%s_%s\" set time_end=%ld where "
 		"time_end=0 and node_name='%s';",
 		mysql_conn->cluster_name, event_table,
 		event_time, node_ptr->name);
+
+	/*
+	 * Reason for "on duplicate": slurmctld will send a time_start based on
+	 * the state of the "node_state" state file. If the the slurmctld is
+	 * "killed" before updating the state file, the slurmctld can send the
+	 * same time_start for the node and cause a "Duplicate entry" error.
+	 * This can particually happen when doing clean starts.
+	 */
 	xstrfmtcat(query,
 		   "insert into \"%s_%s\" "
 		   "(node_name, state, tres, time_start, "
 		   "reason, reason_uid) "
-		   "values ('%s', %u, '%s', %ld, '%s', %u);",
+		   "values ('%s', %u, '%s', %ld, '%s', %u) "
+		   "on duplicate key update time_end=0;",
 		   mysql_conn->cluster_name, event_table,
 		   node_ptr->name, node_ptr->node_state,
 		   node_ptr->tres_str, event_time, my_reason, reason_uid);
-	if (debug_flags & DEBUG_FLAG_DB_EVENT)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_EVENT, mysql_conn->conn, "query\n%s", query);
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 
@@ -1517,8 +1546,7 @@ extern int as_mysql_node_up(mysql_conn_t *mysql_conn,
 		"time_end=0 and node_name='%s';",
 		mysql_conn->cluster_name, event_table,
 		event_time, node_ptr->name);
-	if (debug_flags & DEBUG_FLAG_DB_EVENT)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_EVENT, mysql_conn->conn, "query\n%s", query);
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 	return rc;
@@ -1538,6 +1566,7 @@ extern int as_mysql_fini_ctld(mysql_conn_t *mysql_conn,
 	time_t now = time(NULL);
 	char *query = NULL;
 	bool free_it = false;
+	bool affected_rows = false;
 
 	if (check_connection(mysql_conn) != SLURM_SUCCESS)
 		return ESLURM_DB_CONNECTION;
@@ -1553,15 +1582,19 @@ extern int as_mysql_fini_ctld(mysql_conn_t *mysql_conn,
 		"control_host='%s' && control_port=%u;",
 		cluster_table, now, cluster_rec->name,
 		cluster_rec->control_host, cluster_rec->control_port);
-	if (debug_flags & DEBUG_FLAG_DB_EVENT)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_EVENT, mysql_conn->conn, "query\n%s", query);
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 
 	if (rc != SLURM_SUCCESS)
 		return SLURM_ERROR;
 
-	if (!last_affected_rows(mysql_conn) || !slurmdbd_conf->track_ctld)
+	affected_rows = last_affected_rows(mysql_conn);
+	if (affected_rows)
+		as_mysql_add_feds_to_update_list(mysql_conn);
+
+	if (!affected_rows || !slurmdbd_conf->track_ctld ||
+	    (cluster_rec->flags & CLUSTER_FLAG_EXT))
 		return rc;
 
 	/* If tres is NULL we can get the current number of tres by
@@ -1599,8 +1632,7 @@ extern int as_mysql_fini_ctld(mysql_conn_t *mysql_conn,
 	if (free_it)
 		xfree(cluster_rec->tres_str);
 
-	if (debug_flags & DEBUG_FLAG_DB_EVENT)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_EVENT, mysql_conn->conn, "query\n%s", query);
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 
@@ -1682,22 +1714,19 @@ extern int as_mysql_cluster_tres(mysql_conn_t *mysql_conn,
 		 */
 
 		if (xstrcmp(cluster_nodes, row[1])) {
-			if (debug_flags & DEBUG_FLAG_DB_EVENT)
-				DB_DEBUG(mysql_conn->conn,
-					 "Nodes on the cluster have changed.");
+			DB_DEBUG(DB_EVENT, mysql_conn->conn,
+			         "Nodes on the cluster have changed.");
 			response = ACCOUNTING_NODES_CHANGE_DB;
 		} else
 			response = ACCOUNTING_TRES_CHANGE_DB;
 	} else if (xstrcmp(cluster_nodes, row[1])) {
-		if (debug_flags & DEBUG_FLAG_DB_EVENT)
-			DB_DEBUG(mysql_conn->conn,
-				 "Node names on the cluster have changed.");
+		DB_DEBUG(DB_EVENT, mysql_conn->conn,
+		         "Node names on the cluster have changed.");
 		response = ACCOUNTING_NODES_CHANGE_DB;
 	} else {
-		if (debug_flags & DEBUG_FLAG_DB_EVENT)
-			DB_DEBUG(mysql_conn->conn,
-				 "We have the same TRES and node names as before for %s, no need to update the database.",
-				 mysql_conn->cluster_name);
+		DB_DEBUG(DB_EVENT, mysql_conn->conn,
+		         "We have the same TRES and node names as before for %s, no need to update the database.",
+		         mysql_conn->cluster_name);
 		goto remove_disconnect;
 	}
 
@@ -1715,13 +1744,18 @@ add_it:
 	query = xstrdup_printf(
 		"insert into \"%s_%s\" (cluster_nodes, tres, "
 		"time_start, reason) "
-		"values ('%s', '%s', %ld, 'Cluster Registered TRES');",
+		"values ('%s', '%s', %ld, 'Cluster Registered TRES') "
+		"on duplicate key update time_end=0, tres=VALUES(tres);",
 		mysql_conn->cluster_name, event_table,
 		cluster_nodes, *tres_str_in, event_time);
 
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 
+	if (trigger_reroll(mysql_conn, event_time))
+		debug("Need to reroll usage from %s, cluster %s changes happened before last rollup.",
+		      slurm_ctime2(&event_time), mysql_conn->cluster_name);
+
 	if (rc != SLURM_SUCCESS)
 		goto end_it;
 
