@@ -285,7 +285,7 @@ static int _update_unused_wall(local_resv_usage_t *r_usage, List job_tres,
 		 * With a Flex reservation you can easily have more time than is
 		 * possible.  Just print this debug3 warning if it happens.
 		 */
-		debug3("WARNING: Unused wall is less than zero; this should never happen outside a Flex reservation. Setting it to zero for resv id = %d, start = %ld.",
+		debug3("Unused wall is less than zero; this should never happen outside a Flex reservation. Setting it to zero for resv id = %d, start = %ld.",
 		       r_usage->id, r_usage->orig_start);
 		r_usage->unused_wall = 0;
 	}
@@ -366,18 +366,15 @@ static void _add_tres_2_list(List tres_list, char *tres_str, int seconds)
 
 static void _add_job_alloc_time_to_assoc(List a_tres_list, List j_tres_list)
 {
-	ListIterator itr;
 	local_tres_usage_t *loc_a_tres, *loc_j_tres;
 
 	/*
-	 * NOTE: you can not use list_pop, or list_push
-	 * anywhere either, since as_mysql is
-	 * exporting something of the same type as a macro,
-	 * which messes everything up
+	 * NOTE: You have to use slurm_list_pop here, since
+	 * mysql is exporting something of the same type as a
+	 * macro, which messes everything up
 	 * (my_list.h is the bad boy).
 	 */
-	itr = list_iterator_create(j_tres_list);
-	while ((loc_j_tres = list_next(itr))) {
+	while ((loc_j_tres = slurm_list_pop(j_tres_list))) {
 		if (!(loc_a_tres = list_find_first(
 			      a_tres_list, _find_loc_tres, &loc_j_tres->id))) {
 			/*
@@ -385,17 +382,11 @@ static void _add_job_alloc_time_to_assoc(List a_tres_list, List j_tres_list)
 			 * just transfer it over.
 			 */
 			list_append(a_tres_list, loc_j_tres);
-			list_remove(itr);
 			continue;
 		}
 		loc_a_tres->time_alloc += loc_j_tres->time_alloc;
-		/*
-		 * We are freeing this list right after this might as well
-		 * delete it now.
-		 */
-		list_delete_item(itr);
+		_destroy_local_tres_usage(loc_j_tres);
 	}
-	list_iterator_destroy(itr);
 }
 
 /* This will destroy the *loc_tres given after it is transfered */
@@ -546,7 +537,7 @@ static void _setup_cluster_tres_usage(mysql_conn_t *mysql_conn,
 				      local_tres_usage_t *loc_tres,
 				      char **query)
 {
-	char start_char[20], end_char[20];
+	char start_char[256], end_char[256];
 	uint64_t total_used;
 
 	if (!loc_tres)
@@ -562,9 +553,7 @@ static void _setup_cluster_tres_usage(mysql_conn_t *mysql_conn,
 				    sizeof(start_char));
 		slurm_make_time_str(&curr_end, end_char,
 				    sizeof(end_char));
-		error("We have more allocated time than is "
-		      "possible (%"PRIu64" > %"PRIu64") for "
-		      "cluster %s(%"PRIu64") from %s - %s tres %u",
+		error("We have more allocated time than is possible (%"PRIu64" > %"PRIu64") for cluster %s(%"PRIu64") from %s - %s tres %u (this may happen if oversubscription of resources is allowed without Gang)",
 		      loc_tres->time_alloc, loc_tres->total_time,
 		      cluster_name, loc_tres->count,
 		      start_char, end_char, loc_tres->id);
@@ -583,10 +572,7 @@ static void _setup_cluster_tres_usage(mysql_conn_t *mysql_conn,
 				    sizeof(start_char));
 		slurm_make_time_str(&curr_end, end_char,
 				    sizeof(end_char));
-		error("We have more time than is "
-		      "possible (%"PRIu64"+%"PRIu64"+%"
-		      PRIu64")(%"PRIu64") > %"PRIu64" for "
-		      "cluster %s(%"PRIu64") from %s - %s tres %u",
+		error("We have more time than is possible (%"PRIu64"+%"PRIu64"+%"PRIu64")(%"PRIu64") > %"PRIu64" for cluster %s(%"PRIu64") from %s - %s tres %u (this may happen if oversubscription of resources is allowed without Gang)",
 		      loc_tres->time_alloc, loc_tres->time_down,
 		      loc_tres->time_pd, total_used,
 		      loc_tres->total_time,
@@ -680,7 +666,7 @@ static void _setup_cluster_tres_usage(mysql_conn_t *mysql_conn,
 			   "(creation_time, mod_time, "
 			   "time_start, id_tres, count, "
 			   "alloc_secs, down_secs, pdown_secs, "
-			   "idle_secs, over_secs, resv_secs) "
+			   "idle_secs, over_secs, plan_secs) "
 			   "values (%ld, %ld, %ld, %u, %"PRIu64", "
 			   "%"PRIu64", %"PRIu64", %"PRIu64", "
 			   "%"PRIu64", %"PRIu64", %"PRIu64")",
@@ -732,15 +718,14 @@ static int _process_cluster_usage(mysql_conn_t *mysql_conn,
 		   "pdown_secs=VALUES(pdown_secs), "
 		   "idle_secs=VALUES(idle_secs), "
 		   "over_secs=VALUES(over_secs), "
-		   "resv_secs=VALUES(resv_secs)",
+		   "plan_secs=VALUES(plan_secs)",
 		   now);
 
 	/* Spacing out the inserts here instead of doing them
 	   all at once in the end proves to be faster.  Just FYI
 	   so we don't go testing again and again.
 	*/
-	if (debug_flags & DEBUG_FLAG_DB_USAGE)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s", query);
 	rc = mysql_db_query(mysql_conn, query);
 	xfree(query);
 	if (rc != SLURM_SUCCESS)
@@ -859,7 +844,8 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 						   time_t curr_start,
 						   time_t curr_end,
 						   List resv_usage_list,
-						   List cluster_down_list)
+						   List cluster_down_list,
+						   int dims)
 {
 	local_cluster_usage_t *c_usage = NULL;
 	char *query = NULL;
@@ -867,7 +853,9 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 	MYSQL_ROW row;
 	int i = 0;
 	ListIterator d_itr = NULL;
+	ListIterator r_itr = NULL;
 	local_cluster_usage_t *loc_c_usage;
+	local_resv_usage_t *loc_r_usage;
 
 	char *event_req_inx[] = {
 		"node_name",
@@ -896,7 +884,7 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 	 * state.  We handle those later with the reservations.
 	 */
 	query = xstrdup_printf("select %s from \"%s_%s\" where "
-			       "!(state & %d) && (time_start < %ld "
+			       "!(state & %"PRIu64") && (time_start < %ld "
 			       "&& (time_end >= %ld "
 			       "|| time_end = 0)) "
 			       "order by node_name, time_start",
@@ -905,8 +893,7 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 			       curr_end, curr_start);
 	xfree(event_str);
 
-	if (debug_flags & DEBUG_FLAG_DB_USAGE)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		return NULL;
@@ -915,12 +902,13 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 	xfree(query);
 
 	d_itr = list_iterator_create(cluster_down_list);
+	r_itr = list_iterator_create(resv_usage_list);
 	while ((row = mysql_fetch_row(result))) {
 		time_t row_start = slurm_atoul(row[EVENT_REQ_START]);
 		time_t row_end = slurm_atoul(row[EVENT_REQ_END]);
 		uint16_t state = slurm_atoul(row[EVENT_REQ_STATE]);
 		time_t local_start, local_end;
-		int seconds;
+		int seconds, resv_seconds;
 
 		if (row_start < curr_start)
 			row_start = curr_start;
@@ -983,6 +971,54 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 		if (!c_usage)
 			continue;
 
+		resv_seconds = 0;
+		/*
+		 * Now switch this time from any non-maint
+		 * reservations that may have had the node
+		 * allocated during this time.
+		 */
+		list_iterator_reset(r_itr);
+		while ((loc_r_usage = list_next(r_itr))) {
+			time_t temp_end = row_end;
+			time_t temp_start = row_start;
+			List loc_tres = NULL;
+
+			if (hostlist_find_dims(loc_r_usage->hl,
+					       row[EVENT_REQ_NAME], dims)
+			    < 0)
+				continue;
+
+			if (loc_r_usage->start > temp_start)
+				temp_start = loc_r_usage->start;
+			if (loc_r_usage->end < temp_end)
+				temp_end = loc_r_usage->end;
+			if ((resv_seconds = (temp_end - temp_start)) < 1)
+				continue;
+
+			loc_tres = list_create(_destroy_local_tres_usage);
+
+			_add_tres_time_2_list(loc_tres,
+					      row[EVENT_REQ_TRES],
+					      loc_r_usage->flags &
+					      RESERVE_FLAG_MAINT ?
+					      TIME_PDOWN : TIME_DOWN,
+					      resv_seconds,
+					      0, 0);
+			_add_tres_time_2_list(c_usage->loc_tres,
+					      row[EVENT_REQ_TRES],
+					      loc_r_usage->flags &
+					      RESERVE_FLAG_MAINT ?
+					      TIME_PDOWN : TIME_DOWN,
+					      resv_seconds,
+					      0, 0);
+
+			_remove_job_tres_time_from_cluster(
+				loc_r_usage->loc_tres,
+				loc_tres, resv_seconds);
+
+			FREE_NULL_LIST(loc_tres);
+		}
+
 		local_start = row_start;
 		local_end = row_end;
 
@@ -995,10 +1031,21 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 		if ((seconds = (local_end - local_start)) < 1)
 			continue;
 
-		_add_tres_time_2_list(c_usage->loc_tres,
-				      row[EVENT_REQ_TRES],
-				      TIME_DOWN,
-				      seconds, 0, 0);
+		seconds -= resv_seconds;
+		if (seconds > 0) {
+			if (((state & NODE_STATE_BASE) == NODE_STATE_FUTURE) ||
+			    ((state & NODE_STATE_CLOUD) &&
+			     (state & NODE_STATE_POWERED_DOWN)))
+				_add_tres_time_2_list(c_usage->loc_tres,
+						      row[EVENT_REQ_TRES],
+						      TIME_PDOWN,
+						      seconds, 0, 0);
+			else
+				_add_tres_time_2_list(c_usage->loc_tres,
+						      row[EVENT_REQ_TRES],
+						      TIME_DOWN,
+						      seconds, 0, 0);
+		}
 
 		/*
 		 * Now remove this time if there was a
@@ -1038,7 +1085,7 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 	return c_usage;
 }
 
-extern int _setup_resv_usage(mysql_conn_t *mysql_conn,
+static int _setup_resv_usage(mysql_conn_t *mysql_conn,
 			     char *cluster_name,
 			     time_t curr_start,
 			     time_t curr_end,
@@ -1086,8 +1133,7 @@ extern int _setup_resv_usage(mysql_conn_t *mysql_conn,
 			       resv_str, cluster_name, resv_table,
 			       curr_end, curr_start);
 	xfree(resv_str);
-	if (debug_flags & DEBUG_FLAG_DB_USAGE)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s", query);
 
 	result = mysql_db_query_ret(mysql_conn, query, 0);
 	xfree(query);
@@ -1174,6 +1220,35 @@ extern int _setup_resv_usage(mysql_conn_t *mysql_conn,
 	return SLURM_SUCCESS;
 }
 
+static void _add_planned_time(local_cluster_usage_t *c_usage, time_t job_start,
+			      time_t job_eligible, uint32_t array_pending,
+			      uint32_t row_rcpu)
+{
+	int eligible_start, eligible_end, loc_seconds = 0;
+
+	if (!c_usage || (job_start && (job_start < c_usage->start)))
+		return;
+
+	eligible_start = MAX(job_eligible, c_usage->start);
+	eligible_end = job_start ? MIN(job_start, c_usage->end) : c_usage->end;
+	loc_seconds = (eligible_end - eligible_start);
+
+	if (loc_seconds <= 0)
+		return;
+
+	/*
+	* If we have pending jobs in an array
+	* they haven't been inserted into the
+	* database yet as proper job records,
+	* so handle them here.
+	*/
+	if (array_pending)
+		loc_seconds *= array_pending;
+
+	_add_time_tres(c_usage->loc_tres, TIME_RESV, TRES_CPU,
+		       loc_seconds * (uint64_t) row_rcpu, 0);
+}
+
 extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				  char *cluster_name,
 				  time_t start, time_t end,
@@ -1261,8 +1336,7 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 	/* We need to figure out the dimensions of this cluster */
 	query = xstrdup_printf("select dimensions from %s where name='%s'",
 			       cluster_table, cluster_name);
-	if (debug_flags & DEBUG_FLAG_DB_USAGE)
-		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+	DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s", query);
 	result = mysql_db_query_ret(mysql_conn, query, 0);
 	xfree(query);
 
@@ -1293,10 +1367,9 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 		int last_id = -1;
 		int last_wckeyid = -1;
 
-		if (debug_flags & DEBUG_FLAG_DB_USAGE)
-			DB_DEBUG(mysql_conn->conn,
-				 "%s curr hour is now %ld-%ld",
-				 cluster_name, curr_start, curr_end);
+		DB_DEBUG(DB_USAGE, mysql_conn->conn,
+		         "%s curr hour is now %ld-%ld",
+		         cluster_name, curr_start, curr_end);
 /* 		info("start %s", slurm_ctime2(&curr_start)); */
 /* 		info("end %s", slurm_ctime2(&curr_end)); */
 
@@ -1309,7 +1382,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 		c_usage = _setup_cluster_usage(mysql_conn, cluster_name,
 					       curr_start, curr_end,
 					       resv_usage_list,
-					       cluster_down_list);
+					       cluster_down_list,
+					       dims);
 
 		if (c_usage)
 			xassert(c_usage->loc_tres);
@@ -1326,8 +1400,7 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				       job_str, cluster_name, job_table,
 				       curr_end, curr_start);
 
-		if (debug_flags & DEBUG_FLAG_DB_USAGE)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s", query);
 		if (!(result = mysql_db_query_ret(
 			      mysql_conn, query, 0))) {
 			rc = SLURM_ERROR;
@@ -1491,10 +1564,6 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 
 			/* first figure out the reservation */
 			if (resv_id) {
-				if (seconds <= 0) {
-					_transfer_loc_tres(&loc_tres, a_usage);
-					continue;
-				}
 				/*
 				 * Since we have already added the entire
 				 * reservation as used time on the cluster we
@@ -1520,6 +1589,19 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 					 */
 					if (r_usage->id != resv_id)
 						continue;
+
+					if (r_usage->flags &
+					    RESERVE_FLAG_IGN_JOBS) {
+						_add_planned_time(
+							c_usage,
+							MIN(row_start,
+							    r_usage->end),
+							MAX(row_eligible,
+							    r_usage->start),
+							array_pending,
+							row_rcpu);
+					}
+
 					temp_end = row_end;
 					temp_start = row_start;
 					if (r_usage->start > temp_start)
@@ -1566,17 +1648,7 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				continue;
 			}
 
-			/*
-			 * only record time for the clusters that have
-			 * registered.  This continue should rarely if
-			 * ever happen.
-			 */
-			if (!c_usage) {
-				_transfer_loc_tres(&loc_tres, a_usage);
-				continue;
-			}
-
-			if (row_start && (seconds > 0)) {
+			if (c_usage && row_start && (seconds > 0)) {
 				/* info("%d assoc %d adds " */
 				/*      "(%d)(%d-%d) * %d = %d " */
 				/*      "to %d", */
@@ -1599,44 +1671,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 			 */
 			_transfer_loc_tres(&loc_tres, a_usage);
 
-			/* now reserved time */
-			if (!row_start || (row_start >= c_usage->start)) {
-				int temp_end = row_start;
-				int temp_start = row_eligible;
-				if (c_usage->start > temp_start)
-					temp_start = c_usage->start;
-				if (c_usage->end < temp_end)
-					temp_end = c_usage->end;
-				loc_seconds = (temp_end - temp_start);
-				if (loc_seconds > 0) {
-					/*
-					 * If we have pending jobs in an array
-					 * they haven't been inserted into the
-					 * database yet as proper job records,
-					 * so handle them here.
-					 */
-					if (array_pending)
-						loc_seconds *= array_pending;
-
-					/* info("%d assoc %d reserved " */
-					/*      "(%d)(%d-%d) * %d * %d = %d " */
-					/*      "to %d", */
-					/*      job_id, */
-					/*      assoc_id, */
-					/*      temp_end - temp_start, */
-					/*      temp_end, temp_start, */
-					/*      row_rcpu, */
-					/*      array_pending, */
-					/*      loc_seconds, */
-					/*      row_rcpu); */
-
-					_add_time_tres(c_usage->loc_tres,
-						       TIME_RESV, TRES_CPU,
-						       loc_seconds *
-						       (uint64_t) row_rcpu,
-						       0);
-				}
-			}
+			_add_planned_time(c_usage, row_start, row_eligible,
+					  array_pending, row_rcpu);
 		}
 		mysql_free_result(result);
 
@@ -1696,10 +1732,10 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 						a_usage->id = associd;
 						list_append(assoc_usage_list,
 							    a_usage);
-						last_id = associd;
 						a_usage->loc_tres = list_create(
 							_destroy_local_tres_usage);
 					}
+					last_id = associd;
 
 					_add_time_tres(a_usage->loc_tres,
 						       TIME_ALLOC, loc_tres->id,
@@ -1711,8 +1747,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 		}
 
 		if (query) {
-			if (debug_flags & DEBUG_FLAG_DB_USAGE)
-				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+			DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s",
+			         query);
 			rc = mysql_db_query(mysql_conn, query);
 			xfree(query);
 			if (rc != SLURM_SUCCESS) {
@@ -1751,8 +1787,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 						curr_start, now,
 						a_usage, &query);
 		if (query) {
-			if (debug_flags & DEBUG_FLAG_DB_USAGE)
-				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+			DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s",
+			         query);
 			rc = mysql_db_query(mysql_conn, query);
 			xfree(query);
 			if (rc != SLURM_SUCCESS) {
@@ -1770,8 +1806,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 						curr_start, now,
 						w_usage, &query);
 		if (query) {
-			if (debug_flags & DEBUG_FLAG_DB_USAGE)
-				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+			DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s",
+			         query);
 			rc = mysql_db_query(mysql_conn, query);
 			xfree(query);
 			if (rc != SLURM_SUCCESS) {
@@ -1874,10 +1910,9 @@ extern int as_mysql_nonhour_rollup(mysql_conn_t *mysql_conn,
 
 		curr_end = slurm_mktime(&start_tm);
 
-		if (debug_flags & DEBUG_FLAG_DB_USAGE)
-			DB_DEBUG(mysql_conn->conn,
-				 "curr %s is now %ld-%ld",
-				 unit_name, curr_start, curr_end);
+		DB_DEBUG(DB_USAGE, mysql_conn->conn,
+		         "curr %s is now %ld-%ld",
+		         unit_name, curr_start, curr_end);
 /* 		info("start %s", slurm_ctime2(&curr_start)); */
 /* 		info("end %s", slurm_ctime2(&curr_end)); */
 		query = xstrdup_printf(
@@ -1903,7 +1938,7 @@ extern int as_mysql_nonhour_rollup(mysql_conn_t *mysql_conn,
 			   "insert into \"%s_%s\" (creation_time, "
 			   "mod_time, time_start, id_tres, count, "
 			   "alloc_secs, down_secs, pdown_secs, "
-			   "idle_secs, over_secs, resv_secs) "
+			   "idle_secs, over_secs, plan_secs) "
 			   "select %ld, %ld, "
 			   "%ld, id_tres, @CPU:=MAX(count), "
 			   "@ASUM:=SUM(alloc_secs), "
@@ -1911,14 +1946,14 @@ extern int as_mysql_nonhour_rollup(mysql_conn_t *mysql_conn,
 			   "@PDSUM:=SUM(pdown_secs), "
 			   "@ISUM:=SUM(idle_secs), "
 			   "@OSUM:=SUM(over_secs), "
-			   "@RSUM:=SUM(resv_secs) from \"%s_%s\" where "
+			   "@PSUM:=SUM(plan_secs) from \"%s_%s\" where "
 			   "(time_start < %ld && time_start >= %ld) "
 			   "group by deleted, id_tres "
 			   "on duplicate key update "
 			   "mod_time=%ld, count=@CPU, "
 			   "alloc_secs=@ASUM, down_secs=@DSUM, "
 			   "pdown_secs=@PDSUM, idle_secs=@ISUM, "
-			   "over_secs=@OSUM, resv_secs=@RSUM;",
+			   "over_secs=@OSUM, plan_secs=@PSUM;",
 			   cluster_name,
 			   run_month ? cluster_month_table : cluster_day_table,
 			   now, now, curr_start,
@@ -1945,8 +1980,7 @@ extern int as_mysql_nonhour_rollup(mysql_conn_t *mysql_conn,
 				   wckey_hour_table,
 				   curr_end, curr_start, now);
 		}
-		if (debug_flags & DEBUG_FLAG_DB_USAGE)
-			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+		DB_DEBUG(DB_USAGE, mysql_conn->conn, "query\n%s", query);
 		rc = mysql_db_query(mysql_conn, query);
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
