@@ -2674,14 +2674,21 @@ extern char *acct_storage_p_node_inx(void *db_conn, char *nodes)
 	bitstr_t *node_bitmap;
 	hostlist_iterator_t h_itr;
 
-	if (!nodes || !cluster_hl)
+	if (!nodes)
 		return NULL;
-
 	node_hl = hostlist_create(nodes);
-	node_bitmap = bit_alloc(node_record_count);
 	h_itr = hostlist_iterator_create(node_hl);
 
 	slurm_mutex_lock(&cluster_hl_mutex);
+	if (!cluster_hl) {
+		slurm_mutex_unlock(&cluster_hl_mutex);
+		hostlist_iterator_destroy(h_itr);
+		FREE_NULL_HOSTLIST(node_hl);
+		return NULL;
+	}
+
+	node_bitmap = bit_alloc(hostlist_count(cluster_hl));
+
 	while ((host = hostlist_next(h_itr))) {
 		int loc;
 		if ((loc = hostlist_find(cluster_hl, host)) != -1)
@@ -3433,8 +3440,9 @@ extern void acct_storage_p_send_all(void *db_conn, time_t event_time,
 	debug2("called %s", rpc_num2string(msg_type));
 	switch (msg_type) {
 	case ACCOUNTING_FIRST_REG:
-	case ACCOUNTING_NODES_CHANGE_DB:
 		(void) send_jobs_to_accounting();
+		/* fall through */
+	case ACCOUNTING_NODES_CHANGE_DB:
 		(void) send_resvs_to_accounting(msg_type);
 		/* fall through */
 	case ACCOUNTING_TRES_CHANGE_DB:
