@@ -863,12 +863,6 @@ extern int slurmdbd_agent_send_recv(uint16_t rpc_version,
 
 	xassert(req);
 	xassert(resp);
-	xassert(slurmdbd_conn);
-
-	if (req->conn && (req->conn != slurmdbd_conn))
-		error("We are overriding the connection!!!!!");
-
-	req->conn = slurmdbd_conn;
 
 	/*
 	 * To make sure we can get this to send instead of the agent
@@ -877,8 +871,20 @@ extern int slurmdbd_agent_send_recv(uint16_t rpc_version,
 	 */
 	halt_agent = 1;
 	slurm_mutex_lock(&slurmdbd_lock);
+
 	halt_agent = 0;
 
+	if (!slurmdbd_conn) {
+		slurm_cond_signal(&slurmdbd_cond);
+		slurm_mutex_unlock(&slurmdbd_lock);
+		return ESLURM_DB_CONNECTION_INVALID;
+	}
+
+	if (req->conn && (req->conn != slurmdbd_conn))
+		error("We are overriding the connection!!!!!");
+
+	req->conn = slurmdbd_conn;
+
 	rc = dbd_conn_send_recv_direct(rpc_version, req, resp);
 
 	slurm_cond_signal(&slurmdbd_cond);
