@@ -58,12 +58,12 @@
 
 #include "src/common/slurm_xlator.h"
 #include "src/common/fd.h"
-#include "src/common/slurm_acct_gather_profile.h"
+#include "src/interfaces/acct_gather_profile.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_time.h"
 #include "src/common/xstring.h"
-#include "src/slurmd/common/proctrack.h"
+#include "src/interfaces/proctrack.h"
 #include "hdf5_api.h"
 
 #define HDF5_CHUNK_SIZE 10
@@ -270,7 +270,9 @@ static void _create_directories(void)
 	slash[0] = '\0';
 	hdf5_dir_rel = slash + 1;
 
-	parent_dirfd = open(parent_dir, O_DIRECTORY | O_NOFOLLOW);
+	if ((parent_dirfd = open(parent_dir, O_DIRECTORY | O_NOFOLLOW)) < 0)
+		fatal("Could not open ProfileHDF5Dir parent directory '%s': %m",
+		      parent_dir);
 
 	/*
 	 * Use *at family of syscalls to prevent TOCTOU abuse by working
