@@ -131,25 +131,14 @@ static const char *jwks_key_field = "jwks=";
 
 static data_for_each_cmd_t _build_jwks_keys(data_t *d, void *arg)
 {
-	char *alg, *kid, *n, *e, *key, *x5c, *kty;
+	char *alg, *kid, *n, *e, *key;
 
 	if (!(kid = data_get_string(data_key_get(d, "kid"))))
 		fatal("%s: failed to load kid field", __func__);
 
-	alg = data_get_string(data_key_get(d, "alg"));
-	kty = data_get_string(data_key_get(d, "kty"));
-	x5c = data_get_string(data_key_get(d, "x5c"));
-
-	if (kty && x5c && !xstrcasecmp(kty, "RSA") && x5c[0]) {
-		debug3("x5c for kid %s is %s", kid, x5c);
-
-		data_set_int(data_key_set(d, "slurm-pem-len"), strlen(x5c));
-		data_set_string_own(data_key_set(d, "slurm-pem"), x5c);
-		return DATA_FOR_EACH_CONT;
-	}
-
-	/* Ignore non-RS256 keys in the JWKS */
-	if (xstrcasecmp(alg, "RS256"))
+	/* Ignore non-RS256 keys in the JWKS if algorthim is provided */
+	if ((alg = data_get_string(data_key_get(d, "alg"))) &&
+	    xstrcasecmp(alg, "RS256"))
 		return DATA_FOR_EACH_CONT;
 
 	if (!(e = data_get_string(data_key_get(d, "e"))))
@@ -455,6 +444,9 @@ int auth_p_verify(auth_token_t *cred, char *auth_info)
 		goto fail;
 	}
 
+	jwt_free(jwt);
+	jwt = NULL;
+
 	if (!cred->username)
 		cred->username = username;
 	else if (!xstrcmp(cred->username, username)) {
