@@ -656,6 +656,8 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 	} else if (!xstrcmp(name, "shared") ||
 		   !xstrcmp(name, "oversubscribe")) {
 		lua_pushnumber(L, job_desc->shared);
+	} else if (!xstrcmp(name, "selinux_context")) {
+		lua_pushstring(L, job_desc->selinux_context);
 	} else if (!xstrcmp(name, "site_factor")) {
 		if (job_desc->site_factor == NO_VAL)
 			lua_pushnumber(L, job_desc->site_factor);
@@ -1189,8 +1191,6 @@ static int _part_rec_field_index(lua_State *L)
 static bool _user_can_use_part(uint32_t user_id, uint32_t submit_uid,
 			       part_record_t *part_ptr)
 {
-	int i;
-
 	if (user_id == 0) {
 		if (part_ptr->flags & PART_FLAG_NO_ROOT)
 			return false;
@@ -1200,10 +1200,10 @@ static bool _user_can_use_part(uint32_t user_id, uint32_t submit_uid,
 	if ((part_ptr->flags & PART_FLAG_ROOT_ONLY) && (submit_uid != 0))
 		return false;
 
-	if (part_ptr->allow_uids == NULL)
+	if (!part_ptr->allow_uids_cnt)
 		return true;	/* No user ID filters */
 
-	for (i=0; part_ptr->allow_uids[i]; i++) {
+	for (int i = 0; i < part_ptr->allow_uids_cnt; i++) {
 		if (user_id == part_ptr->allow_uids[i])
 			return true;
 	}
