@@ -450,7 +450,8 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 					 bool enforce_binding,
 					 bool first_pass,
 					 bitstr_t *avail_core,
-					 char *node_name)
+					 char *node_name,
+					 uint16_t cr_type)
 {
 	ListIterator sock_gres_iter;
 	sock_gres_t *sock_gres;
@@ -493,6 +494,19 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 		int min_core_cnt, req_cores, rem_sockets, req_sock_cnt = 0;
 		int threads_per_core;
 
+		/*
+		 * sock_gres->total_cnt is a value used by gres_sched_add
+		 * it may be decreased by gres_select_filter_sock_core
+		 * in first_pass, but in 2nd pass we should start
+		 * from the value set by gres_select_filter_remove_unusable
+		 */
+		if (first_pass && !sock_gres->total_cnt_before_filter)
+			sock_gres->total_cnt_before_filter =
+				sock_gres->total_cnt;
+		else
+			sock_gres->total_cnt =
+				sock_gres->total_cnt_before_filter;
+
 		if (mc_ptr->threads_per_core)
 			threads_per_core =
 				MIN(cpus_per_core,
@@ -555,6 +569,9 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 			 (gres_js->ntasks_per_gres != NO_VAL16)) {
 			cpus_per_gres = gres_js->ntasks_per_gres *
 				mc_ptr->cpus_per_task;
+		} else if (gres_js->def_cpus_per_gres) {
+			cpus_per_gres = gres_js->def_cpus_per_gres;
+			has_cpus_per_gres = true;
 		} else if (first_pass && mc_ptr->ntasks_per_job &&
 			   (mc_ptr->ntasks_per_job != NO_VAL) &&
 			   gres_js->gres_per_job) {
@@ -570,9 +587,6 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 			 * allocate because of overestimate this won't be an
 			 * issue since it's only in first_pass.
 			 */
-		} else if (gres_js->def_cpus_per_gres) {
-			cpus_per_gres = gres_js->def_cpus_per_gres;
-			has_cpus_per_gres = true;
 		}
 
 		/* Filter out unusable GRES by socket */
@@ -618,7 +632,6 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 			    (tot_gres_sock == 0)) {
 				/*
 				 * Insufficient GRES on this socket
-				 * GRES removed here won't be used in 2nd pass
 				 */
 				if (((gres_js->gres_per_socket >
 				      tot_gres_sock) ||
@@ -633,17 +646,16 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 					for (int c = cores_per_socket - 1;
 					     c >= 0; c--) {
 						int i = (s * cores_per_socket) + c;
+						int cnt;
 						if (!bit_test(avail_core, i))
 							continue;
 						bit_clear(avail_core, i);
 
 						avail_cores_per_sock[s]--;
-						if (bit_set_count(avail_core) *
-						    cpus_per_core <
-						    *avail_cpus) {
-							*avail_cpus -=
-								cpus_per_core;
-						}
+						cnt = bit_set_count(avail_core);
+						cnt *= cpus_per_core;
+						if (cnt < *avail_cpus)
+							*avail_cpus = cnt;
 						if (--tot_core_cnt <=
 						    min_core_cnt)
 							break;
@@ -701,6 +713,11 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 						   max_tasks);
 		}
 
+		if (gres_js->ntasks_per_gres) {
+			max_tasks = cnt_avail_total * gres_js->ntasks_per_gres;
+			*max_tasks_this_node = MIN(*max_tasks_this_node,
+						   max_tasks);
+		}
 		/*
 		 * min_tasks_this_node and max_tasks_this_node must be multiple
 		 * of original min_tasks_this_node value. This is to support
@@ -726,13 +743,16 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 					continue;
 				for (int c = cores_per_socket - 1; c >= 0; c--) {
 					int i = (s * cores_per_socket) + c;
+					int cnt;
 					if (!bit_test(avail_core, i))
 						continue;
 					bit_clear(avail_core, i);
-					if (bit_set_count(avail_core) *
-					    cpus_per_core < *avail_cpus) {
-						*avail_cpus -= cpus_per_core;
-					}
+
+					cnt = bit_set_count(avail_core);
+					cnt *= cpus_per_core;
+					if (cnt < *avail_cpus)
+						*avail_cpus = cnt;
+
 					avail_cores_tot--;
 					avail_cores_per_sock[s]--;
 				}
@@ -887,7 +907,8 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 		 * up to required number of cores based on max_tasks_this_node.
 		 * In case of enforce-binding those are already cleared.
 		 */
-		if ((avail_cores_tot > req_cores) &&
+		if (!(cr_type & CR_SOCKET) &&
+		    (avail_cores_tot > req_cores) &&
 		    !enforce_binding && !first_pass &&
 		    (req_sock_cnt != sockets)) {
 			for (int s = 0; s < sockets; s++) {
@@ -897,13 +918,15 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 					continue;
 				for (int c = cores_per_socket - 1; c >= 0; c--) {
 					int i = (s * cores_per_socket) + c;
+					int cnt;
 					if (!bit_test(avail_core, i))
 						continue;
 					bit_clear(avail_core, i);
-					if (bit_set_count(avail_core) *
-					    cpus_per_core < *avail_cpus) {
-						*avail_cpus -= cpus_per_core;
-					}
+					cnt = bit_set_count(avail_core);
+					cnt *= cpus_per_core;
+					if (cnt < *avail_cpus)
+						*avail_cpus = cnt;
+
 					avail_cores_tot--;
 					avail_cores_per_sock[s]--;
 					if (avail_cores_tot == req_cores)
@@ -917,7 +940,8 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 		 * spread them out so that every socket has some cores
 		 * available to use with the nearby GRES that we do need.
 		 */
-		while (req_sock_cnt && (avail_cores_tot > req_cores)) {
+		while (!(cr_type & CR_SOCKET) &&
+		       (req_sock_cnt && (avail_cores_tot > req_cores))) {
 			int full_socket = -1;
 			for (int s = 0; s < sockets; s++) {
 				if (avail_cores_tot == req_cores)
@@ -935,13 +959,15 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
 				break;
 			for (int c = cores_per_socket - 1; c >= 0; c--) {
 				int i = (full_socket * cores_per_socket) + c;
+				int cnt;
 				if (!bit_test(avail_core, i))
 					continue;
 				bit_clear(avail_core, i);
-				if (bit_set_count(avail_core) * cpus_per_core <
-				    *avail_cpus) {
-					*avail_cpus -= cpus_per_core;
-				}
+				cnt = bit_set_count(avail_core);
+				cnt *= cpus_per_core;
+				if (cnt < *avail_cpus)
+					*avail_cpus = cnt;
+
 				avail_cores_per_sock[full_socket]--;
 				avail_cores_tot--;
 				break;
@@ -1000,8 +1026,7 @@ extern void gres_select_filter_sock_core(gres_mc_data_t *mc_ptr,
  */
 static void _pick_specific_topo(struct job_resources *job_res, int node_inx,
 				int job_node_inx, sock_gres_t *sock_gres,
-				uint32_t job_id, gres_mc_data_t *tres_mc_ptr,
-				uint32_t **tasks_per_node_socket)
+				uint32_t job_id, gres_mc_data_t *tres_mc_ptr)
 {
 	int core_offset;
 	uint16_t sock_cnt = 0, cores_per_socket_cnt = 0;
@@ -1009,12 +1034,22 @@ static void _pick_specific_topo(struct job_resources *job_res, int node_inx,
 	gres_job_state_t *gres_js;
 	gres_node_state_t *gres_ns;
 	int *used_sock = NULL, alloc_gres_cnt = 0;
-	uint64_t gres_per_bit, gres_per_node;
+	uint64_t gres_per_node = 0;
 	bool use_busy_dev = gres_use_busy_dev(sock_gres->gres_state_node, 0);
 
 	gres_js = sock_gres->gres_state_job->gres_data;
-	gres_per_node = gres_per_bit = gres_js->gres_per_node ?
-		gres_js->gres_per_node : gres_js->gres_per_task;
+
+	/* Determine how many gres are required on this node */
+	if (gres_js->gres_per_node) {
+		gres_per_node = gres_js->gres_per_node;
+	} else if (gres_js->gres_per_task && job_res->tasks_per_node) {
+		gres_per_node = gres_js->gres_per_task *
+				job_res->tasks_per_node[job_node_inx];
+	} else {
+		error("%s: Invalid job resource allocation. No tasks allocated on nodes.",
+		      __func__);
+		return;
+	}
 
 	gres_ns = sock_gres->gres_state_node->gres_data;
 	rc = get_job_resources_cnt(job_res, job_node_inx, &sock_cnt,
@@ -1058,14 +1093,8 @@ static void _pick_specific_topo(struct job_resources *job_res, int node_inx,
 	 */
 	for (s = -1;	/* Socket == - 1 if GRES avail from any socket */
 	     (s < sock_cnt) && (alloc_gres_cnt == 0); s++) {
-		gres_per_node = gres_per_bit;
 		if ((s >= 0) && !used_sock[s])
 			continue;
-		if (gres_js->gres_per_task) {
-			gres_per_node *= tasks_per_node_socket[node_inx][s];
-			if (!gres_per_node)
-				continue;
-		}
 
 		for (t = 0; t < gres_ns->topo_cnt; t++) {
 			if (use_busy_dev &&
@@ -2313,8 +2342,7 @@ extern int gres_select_filter_select_and_set(List *sock_gres_list,
 				/* gres/mps: select specific topo bit for job */
 				_pick_specific_topo(job_res, i, node_inx,
 						    sock_gres, job_id,
-						    tres_mc_ptr,
-					       tasks_per_node_socket);
+						    tres_mc_ptr);
 			} else if (gres_js->gres_per_node) {
 				_set_node_bits(job_res, i, node_inx,
 					       sock_gres, job_id, tres_mc_ptr);
