@@ -2,7 +2,7 @@
  *  switch_cray_aries.c - Library for managing a switch on a Cray/Aries system.
  *****************************************************************************
  *  Copyright (C) 2013 SchedMD LLC
- *  Copyright 2013 Cray Inc. All Rights Reserved.
+ *  Copyright 2013 Hewlett Packard Enterprise Development LP
  *  Written by Danny Auble <da@schedmd.com>
  *
  *  This file is part of Slurm, a resource management program.
@@ -56,8 +56,9 @@
 #include "switch_cray_aries.h"
 #include "slurm/slurm.h"
 #include "slurm/slurm_errno.h"
+#include "src/interfaces/gres.h"
 #include "src/common/pack.h"
-#include "src/common/gres.h"
+#include "src/interfaces/select.h"
 
 #ifdef HAVE_NATIVE_CRAY
 #include <job.h> /* Cray's job module component */
@@ -66,8 +67,6 @@
 #define SWITCH_BUF_SIZE (PORT_CNT + 128)
 #define SWITCH_CRAY_STATE_VERSION "PROTOCOL_VERSION"
 
-uint64_t debug_flags = 0;
-
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
 static bool lustre_no_flush = false;
 #endif
@@ -108,7 +107,6 @@ const uint32_t plugin_id      = SWITCH_PLUGIN_CRAY;
 int init(void)
 {
 	debug("%s loaded.", plugin_name);
-	debug_flags = slurm_get_debug_flags();
 
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
 	start_lease_extender();
@@ -127,7 +125,6 @@ int fini(void)
 
 extern int switch_p_reconfig(void)
 {
-	debug_flags = slurm_get_debug_flags();
 	return SLURM_SUCCESS;
 }
 
@@ -184,7 +181,7 @@ extern int switch_p_alloc_jobinfo(
 
 extern int switch_p_build_jobinfo(switch_jobinfo_t *switch_job,
 				  slurm_step_layout_t *step_layout,
-				  char *network)
+				  step_record_t *step_ptr)
 {
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
 	int rc, cnt = 0;
@@ -217,9 +214,12 @@ extern int switch_p_build_jobinfo(switch_jobinfo_t *switch_job,
 	rc = lease_cookies(job, nodes, step_layout->node_cnt);
 
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 
+	select_g_select_jobinfo_get(step_ptr->select_jobinfo,
+				    SELECT_JOBDATA_NETWORK,
+				    &job->access);
 	xfree(nodes);
 	if (rc != SLURM_SUCCESS) {
 		return rc;
@@ -303,13 +303,13 @@ extern void switch_p_free_jobinfo(switch_jobinfo_t *switch_job)
 endit:
 	xfree(job);
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 
 	return;
 }
 
-extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, Buf buffer,
+extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, buf_t *buffer,
 				 uint16_t protocol_version)
 {
 	slurm_cray_jobinfo_t *job = (slurm_cray_jobinfo_t *) switch_job;
@@ -327,12 +327,19 @@ extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, Buf buffer,
 
 	xassert(job->magic == CRAY_JOBINFO_MAGIC);
 
-	if (debug_flags & DEBUG_FLAG_SWITCH) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH) {
 		CRAY_INFO("switch_jobinfo_t contents:");
 		print_jobinfo(job);
 	}
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+		pack32(job->magic, buffer);
+		pack32(job->num_cookies, buffer);
+		packstr_array(job->cookies, job->num_cookies, buffer);
+		pack32_array(job->cookie_ids, job->num_cookies, buffer);
+		pack64(job->apid, buffer);
+		pack16(job->access, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack32(job->magic, buffer);
 		pack32(job->num_cookies, buffer);
 		packstr_array(job->cookies, job->num_cookies, buffer);
@@ -343,7 +350,7 @@ extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, Buf buffer,
 	return 0;
 }
 
-extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, Buf buffer,
+extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, buf_t *buffer,
 				   uint16_t protocol_version)
 {
 	uint32_t num_cookies;
@@ -359,7 +366,33 @@ extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, Buf buffer,
 	job = xmalloc(sizeof(slurm_cray_jobinfo_t));
 	*switch_job = (switch_jobinfo_t *)job;
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+		safe_unpack32(&job->magic, buffer);
+
+		if (job->magic == CRAY_NULL_JOBINFO_MAGIC) {
+			CRAY_DEBUG("Nothing to unpack");
+			return SLURM_SUCCESS;
+		}
+
+		xassert(job->magic == CRAY_JOBINFO_MAGIC);
+		safe_unpack32(&(job->num_cookies), buffer);
+		safe_unpackstr_array(&(job->cookies), &num_cookies, buffer);
+		if (num_cookies != job->num_cookies) {
+			CRAY_ERR("Wrong number of cookies received."
+				 " Expected: %" PRIu32 "Received: %" PRIu32,
+				 job->num_cookies, num_cookies);
+			goto unpack_error;
+		}
+		safe_unpack32_array(&(job->cookie_ids), &num_cookies, buffer);
+		if (num_cookies != job->num_cookies) {
+			CRAY_ERR("Wrong number of cookie IDs received."
+				 " Expected: %" PRIu32 "Received: %" PRIu32,
+				 job->num_cookies, num_cookies);
+			goto unpack_error;
+		}
+		safe_unpack64(&job->apid, buffer);
+		safe_unpack16(&job->access, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&job->magic, buffer);
 
 		if (job->magic == CRAY_NULL_JOBINFO_MAGIC) {
@@ -395,7 +428,7 @@ extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, Buf buffer,
 	track_cookies(job);
 #endif
 
-	if (debug_flags & DEBUG_FLAG_SWITCH) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH) {
 		CRAY_INFO("Unpacked jobinfo");
 		print_jobinfo(job);
 	}
@@ -411,52 +444,26 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-extern void switch_p_print_jobinfo(FILE *fp, switch_jobinfo_t *jobinfo)
-{
-	return;
-}
-
-extern char *switch_p_sprint_jobinfo(switch_jobinfo_t *switch_jobinfo,
-				     char *buf, size_t size)
-{
-	if (buf && size) {
-		buf[0] = '\0';
-		return buf;
-	}
-
-	return NULL ;
-}
-
 /*
  * switch functions for job initiation
  */
-extern int switch_p_node_init(void)
+
+extern int switch_p_job_preinit(stepd_step_rec_t *step)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_node_fini(void)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_job_preinit(switch_jobinfo_t *jobinfo)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_job_init(stepd_step_rec_t *job)
+extern int switch_p_job_init(stepd_step_rec_t *step)
 {
 
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
-	slurm_cray_jobinfo_t *sw_job = job->switch_job ?
-		(slurm_cray_jobinfo_t *)job->switch_job->data : NULL;
+	slurm_cray_jobinfo_t *sw_job = step->switch_job ?
+		(slurm_cray_jobinfo_t *)step->switch_job->data : NULL;
 	int rc, num_ptags;
-	char *launch_params;
 	int exclusive = 0, mem_scaling = 100, cpu_scaling = 100;
 	int *ptags = NULL;
 	char *err_msg = NULL;
-	uint64_t cont_id = job->cont_id;
+	uint64_t cont_id = step->cont_id;
 	alpsc_peInfo_t alpsc_pe_info = {-1, -1, -1, -1, NULL, NULL, NULL};
 	int cmd_index = 0;
 #ifdef HAVE_NATIVE_CRAY
@@ -467,41 +474,37 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	uint32_t jobid;
 #endif
 
-#if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
-	char *npc = "none";
-	int access = ALPSC_NET_PERF_CTR_NONE;
-#endif
 	DEF_TIMERS;
 
 	START_TIMER;
 
 #ifdef HAVE_CRAY_NETWORK
 	/* No PAGG job containers; uid used instead to configure network */
-	cont_id = (uint64_t)job->uid;
+	cont_id = (uint64_t)step->uid;
 #endif
 
 	if (!sw_job || (sw_job->magic == CRAY_NULL_JOBINFO_MAGIC)) {
-		CRAY_DEBUG("job->switch_job was NULL");
+		CRAY_DEBUG("step->switch_job was NULL");
 		return SLURM_SUCCESS;
 	}
 
-	xassert(job->msg);
+	xassert(step->msg);
 	xassert(sw_job->magic == CRAY_JOBINFO_MAGIC);
 
 #ifdef HAVE_NATIVE_CRAY
 	// Attach to the cncu container
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		jobid = step->het_job_id;
 	else
-		jobid = job->jobid;
-	rc = alpsc_attach_cncu_container(&err_msg, jobid, job->cont_id);
+		jobid = step->step_id.job_id;
+	rc = alpsc_attach_cncu_container(&err_msg, jobid, step->cont_id);
 	ALPSC_CN_DEBUG("alpsc_attach_cncu_container");
 	if (rc != 1) {
 		return SLURM_ERROR;
 	}
 
 	// Create the apid directory
-	rc = create_apid_dir(sw_job->apid, job->uid, job->gid);
+	rc = create_apid_dir(sw_job->apid, step->uid, step->gid);
 	if (rc != SLURM_SUCCESS) {
 		return rc;
 	}
@@ -518,7 +521,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	/*
 	 * Fill in the alpsc_pe_info structure
 	 */
-	rc = build_alpsc_pe_info(job, &alpsc_pe_info, &cmd_index);
+	rc = build_alpsc_pe_info(step, &alpsc_pe_info, &cmd_index);
 	if (rc != SLURM_SUCCESS) {
 		return rc;
 	}
@@ -529,8 +532,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	 * Cray shmem still uses the network, even when it's using only one
 	 * node, so we must always configure the network.
 	 */
-	launch_params = slurm_get_launch_params();
-	if (launch_params && strstr(launch_params, "cray_net_exclusive")) {
+	if (xstrstr(slurm_conf.launch_params, "cray_net_exclusive")) {
 		/*
 		 * Grant exclusive access and all aries resources to the job.
 		 * Not recommended if you may run multiple steps within
@@ -543,11 +545,10 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 		 */
 		exclusive = 1;
 	}
-	if (launch_params && strstr(launch_params, "lustre_no_flush")) {
+	if (xstrstr(slurm_conf.launch_params, "lustre_no_flush")) {
 		/* Lustre cache flush can cause job bus errors, see bug 4309 */
 		lustre_no_flush = true;
 	}
-	xfree(launch_params);
 
 	if (!exclusive) {
 		/*
@@ -555,16 +556,16 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 		 * non-exclusive jobs.
 		 */
 
-		cpu_scaling = get_cpu_scaling(job);
+		cpu_scaling = get_cpu_scaling(step);
 		if (cpu_scaling == -1)
 			return SLURM_ERROR;
 
-		mem_scaling = get_mem_scaling(job);
+		mem_scaling = get_mem_scaling(step);
 		if (mem_scaling == -1)
 			return SLURM_ERROR;
 	}
 
-	if (debug_flags & DEBUG_FLAG_SWITCH) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH) {
 		CRAY_INFO("Network Scaling: Exclusive %d CPU %d Memory %d",
 			  exclusive, cpu_scaling, mem_scaling);
 	}
@@ -592,7 +593,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
 	// Write the IAA file
-	rc = write_iaa_file(job, sw_job, sw_job->ptags, sw_job->num_ptags,
+	rc = write_iaa_file(step, sw_job, sw_job->ptags, sw_job->num_ptags,
 			    &alpsc_pe_info);
 	if (rc != SLURM_SUCCESS) {
 		free_alpsc_pe_info(&alpsc_pe_info);
@@ -605,18 +606,13 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	 * If there is reserved access to network performance counters,
 	 * configure the appropriate access permission in the kernel.
 	 */
-	access = ALPSC_NET_PERF_CTR_NONE;
-	select_g_select_jobinfo_get(job->msg->select_jobinfo,
-		SELECT_JOBDATA_NETWORK, &npc);
-	CRAY_DEBUG("network performance counters SELECT_JOBDATA_NETWORK %s",
-		npc);
-	if (xstrcasecmp(npc, "system") == 0) {
-		access = ALPSC_NET_PERF_CTR_SYSTEM;
-	} else if (xstrcasecmp(npc, "blade") == 0) {
-		access = ALPSC_NET_PERF_CTR_BLADE;
-	}
-	if (access != ALPSC_NET_PERF_CTR_NONE) {
-		rc = alpsc_set_perf_ctr_perms(&err_msg, job->cont_id, access);
+	CRAY_DEBUG("network performance counters SELECT_JOBDATA_NETWORK %u",
+		   sw_job->access);
+
+	if ((sw_job->access != ALPSC_NET_PERF_CTR_NONE) &&
+	    (sw_job->access != NO_VAL16)) {
+		rc = alpsc_set_perf_ctr_perms(&err_msg, step->cont_id,
+					      (int)sw_job->access);
 		ALPSC_CN_DEBUG("alpsc_set_perf_ctr_perms");
 		if (rc != 1) {
 			free_alpsc_pe_info(&alpsc_pe_info);
@@ -674,7 +670,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	/*
 	 * Write some environment variables used by LLI and PMI
 	 */
-	rc = set_job_env(job, sw_job);
+	rc = set_job_env(step, sw_job);
 	if (rc != SLURM_SUCCESS)
 		return rc;
 
@@ -683,11 +679,11 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 	 * Query the generic resources to see if the GPU should be allocated
 	 */
 
-	rc = gres_get_step_info(job->step_gres_list, "gpu", 0,
+	rc = gres_get_step_info(step->step_gres_list, "gpu", 0,
 				GRES_STEP_DATA_COUNT, &gpu_cnt);
 	CRAY_INFO("gres_cnt: %d %"PRIu64, rc, gpu_cnt);
 	if (gpu_cnt > 0)
-		setup_gpu(job);
+		setup_gpu(step);
 
 	/*
 	 * Set the Job's APID
@@ -696,7 +692,7 @@ extern int switch_p_job_init(stepd_step_rec_t *job)
 #endif
 
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 
@@ -713,13 +709,13 @@ extern void switch_p_job_suspend_info_get(switch_jobinfo_t *jobinfo,
 {
 	return;
 }
-extern void switch_p_job_suspend_info_pack(void *suspend_info, Buf buffer,
+extern void switch_p_job_suspend_info_pack(void *suspend_info, buf_t *buffer,
 					   uint16_t protocol_version)
 {
 	return;
 }
 
-extern int switch_p_job_suspend_info_unpack(void **suspend_info, Buf buffer,
+extern int switch_p_job_suspend_info_unpack(void **suspend_info, buf_t *buffer,
 					    uint16_t protocol_version)
 {
 	return SLURM_SUCCESS;
@@ -769,18 +765,18 @@ extern int switch_p_job_fini(switch_jobinfo_t *jobinfo)
 #endif
 
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_postfini(stepd_step_rec_t *job)
+extern int switch_p_job_postfini(stepd_step_rec_t *step)
 {
 #if defined(HAVE_NATIVE_CRAY) || defined(HAVE_CRAY_NETWORK)
 	int rc;
 	char *err_msg = NULL;
-	uid_t pgid = job->jmgr_pid;
+	uid_t pgid = step->jmgr_pid;
 #ifdef HAVE_NATIVE_CRAY
         uint64_t gpu_cnt = 0;
 #endif
@@ -788,8 +784,8 @@ extern int switch_p_job_postfini(stepd_step_rec_t *job)
 
 	START_TIMER;
 
-	if (!job->switch_job) {
-		CRAY_DEBUG("job->switch_job was NULL");
+	if (!step->switch_job) {
+		CRAY_DEBUG("step->switch_job was NULL");
 	}
 
 	/*
@@ -799,8 +795,8 @@ extern int switch_p_job_postfini(stepd_step_rec_t *job)
 		CRAY_DEBUG("Sending SIGKILL to pgid %lu", (unsigned long) pgid);
 		kill(-pgid, SIGKILL);
 	} else
-		CRAY_INFO("Job %u.%u: Bad pid value %lu",
-			  job->jobid, job->stepid, (unsigned long) pgid);
+		CRAY_INFO("%ps: Bad pid value %lu",
+			  &step->step_id, (unsigned long) pgid);
 	/*
 	 * Clean-up
 	 *
@@ -812,10 +808,10 @@ extern int switch_p_job_postfini(stepd_step_rec_t *job)
 
 #ifdef HAVE_NATIVE_CRAY
 	// Set the proxy back to the default state.
-	rc = gres_get_step_info(job->step_gres_list, "gpu", 0,
+	rc = gres_get_step_info(step->step_gres_list, "gpu", 0,
 				GRES_STEP_DATA_COUNT, &gpu_cnt);
 	if (gpu_cnt > 0) {
-		reset_gpu(job);
+		reset_gpu(step);
 	}
 #endif
 	if (!lustre_no_flush) {
@@ -838,7 +834,7 @@ extern int switch_p_job_postfini(stepd_step_rec_t *job)
 	}
 
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 	return SLURM_SUCCESS;
@@ -859,53 +855,6 @@ extern int switch_p_get_jobinfo(switch_jobinfo_t *switch_job, int key,
 	return SLURM_ERROR;
 }
 
-/*
- * node switch state monitoring functions
- * required for IBM Federation switch
- */
-extern int switch_p_clear_node_state(void)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_alloc_node_info(switch_node_info_t **switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_build_node_info(switch_node_info_t *switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_pack_node_info(switch_node_info_t *switch_node, Buf buffer,
-				   uint16_t protocol_version)
-{
-	return 0;
-}
-
-extern int switch_p_unpack_node_info(switch_node_info_t **switch_node,
-				     Buf buffer, uint16_t protocol_version)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_free_node_info(switch_node_info_t **switch_node)
-{
-	return SLURM_SUCCESS;
-}
-
-extern char*switch_p_sprintf_node_info(switch_node_info_t *switch_node,
-				       char *buf, size_t size)
-{
-	if (buf && size) {
-		buf[0] = '\0';
-		return buf;
-	}
-
-	return NULL ;
-}
-
 extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo,
 				      char *nodelist)
 {
@@ -921,7 +870,7 @@ extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo,
 		return SLURM_SUCCESS;
 	}
 
-	if (debug_flags & DEBUG_FLAG_SWITCH) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH) {
 		CRAY_INFO("switch_p_job_step_complete");
 	}
 
@@ -931,80 +880,52 @@ extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo,
 		return rc;
 	}
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_part_comp(switch_jobinfo_t *jobinfo,
-				       char *nodelist)
-{
-	return SLURM_SUCCESS;
-}
-
-extern bool switch_p_part_comp(void)
-{
-	return false;
-}
-
 extern int switch_p_job_step_allocated(switch_jobinfo_t *jobinfo,
 				       char *nodelist)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_slurmctld_init(void)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_slurmd_init(void)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_slurmd_step_init(void)
-{
-	return SLURM_SUCCESS;
-}
-
 /*
  * Functions for suspend/resume support
  */
-extern int switch_p_job_step_pre_suspend(stepd_step_rec_t *job)
+extern int switch_p_job_step_pre_suspend(stepd_step_rec_t *step)
 {
 #if _DEBUG
-	info("switch_p_job_step_pre_suspend(job %u.%u)",
-		job->jobid, job->stepid);
+	info("switch_p_job_step_pre_suspend(%ps)", &job->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
-	slurm_cray_jobinfo_t *jobinfo = job->switch_job ?
-		(slurm_cray_jobinfo_t *)job->switch_job->data : NULL;
+	slurm_cray_jobinfo_t *jobinfo = step->switch_job ?
+		(slurm_cray_jobinfo_t *)step->switch_job->data : NULL;
 	char *err_msg = NULL;
 	int rc;
 	DEF_TIMERS;
 
 	START_TIMER;
 
-	rc = alpsc_pre_suspend(&err_msg, job->cont_id, jobinfo->ptags,
+	rc = alpsc_pre_suspend(&err_msg, step->cont_id, jobinfo->ptags,
 			       jobinfo->num_ptags, SUSPEND_TIMEOUT_MSEC);
 	ALPSC_CN_DEBUG("alpsc_pre_suspend");
 	if (rc != 1) {
 		return SLURM_ERROR;
 	}
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_post_suspend(stepd_step_rec_t *job)
+extern int switch_p_job_step_post_suspend(stepd_step_rec_t *step)
 {
 #if _DEBUG
-	info("switch_p_job_step_post_suspend(job %u.%u)",
-		job->jobid, job->stepid);
+	info("switch_p_job_step_post_suspend(%ps)", &step->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
 	char *err_msg = NULL;
@@ -1013,51 +934,49 @@ extern int switch_p_job_step_post_suspend(stepd_step_rec_t *job)
 
 	START_TIMER;
 
-	rc = alpsc_post_suspend(&err_msg, job->cont_id);
+	rc = alpsc_post_suspend(&err_msg, step->cont_id);
 	ALPSC_CN_DEBUG("alpsc_post_suspend");
 	if (rc != 1) {
 		return SLURM_ERROR;
 	}
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_pre_resume(stepd_step_rec_t *job)
+extern int switch_p_job_step_pre_resume(stepd_step_rec_t *step)
 {
 #if _DEBUG
-	info("switch_p_job_step_pre_resume(job %u.%u)",
-		job->jobid, job->stepid);
+	info("switch_p_job_step_pre_resume(%ps)", &step->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
-	slurm_cray_jobinfo_t *jobinfo = job->switch_job ?
-		(slurm_cray_jobinfo_t *)job->switch_job->data : NULL;
+	slurm_cray_jobinfo_t *jobinfo = step->switch_job ?
+		(slurm_cray_jobinfo_t *)step->switch_job->data : NULL;
 	char *err_msg = NULL;
 	int rc;
 	DEF_TIMERS;
 
 	START_TIMER;
 
-	rc = alpsc_pre_resume(&err_msg, job->cont_id, jobinfo->ptags,
+	rc = alpsc_pre_resume(&err_msg, step->cont_id, jobinfo->ptags,
 			       jobinfo->num_ptags);
 	ALPSC_CN_DEBUG("alpsc_pre_resume");
 	if (rc != 1) {
 		return SLURM_ERROR;
 	}
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_post_resume(stepd_step_rec_t *job)
+extern int switch_p_job_step_post_resume(stepd_step_rec_t *step)
 {
 #if _DEBUG
-	info("switch_p_job_step_post_resume(job %u.%u)",
-		job->jobid, job->stepid);
+	info("switch_p_job_step_post_resume(%ps)", &step->step_id);
 #endif
 #if defined(HAVE_NATIVE_CRAY) && !defined(HAVE_CRAY_NETWORK)
 	char *err_msg = NULL;
@@ -1066,14 +985,19 @@ extern int switch_p_job_step_post_resume(stepd_step_rec_t *job)
 
 	START_TIMER;
 
-	rc = alpsc_post_resume(&err_msg, job->cont_id);
+	rc = alpsc_post_resume(&err_msg, step->cont_id);
 	ALPSC_CN_DEBUG("alpsc_post_resume");
 	if (rc != 1) {
 		return SLURM_ERROR;
 	}
 	END_TIMER;
-	if (debug_flags & DEBUG_FLAG_TIME_CRAY)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_TIME_CRAY)
 		INFO_LINE("call took: %s", TIME_STR);
 #endif
 	return SLURM_SUCCESS;
 }
+
+extern void switch_p_job_complete(uint32_t job_id)
+{
+	return;
+}
