@@ -188,6 +188,9 @@ int main(int argc, char **argv)
 	slurm_init(NULL);
 	log_init(xbasename(argv[0]), logopt, 0, NULL);
 
+	if (cli_filter_init() != SLURM_SUCCESS)
+		fatal("failed to initialize cli_filter plugin");
+
 	argvzero = argv[0];
 	_set_exit_code();
 
@@ -514,6 +517,12 @@ int main(int argc, char **argv)
 		while ((desc = list_next(iter_req))) {
 			alloc = list_next(iter_resp);
 
+			/*
+			 * Set JOB_NTASKS_SET to make SLURM_NTASKS get set when
+			 * --ntasks-per-node is requested.
+			 */
+			if (desc->ntasks_per_node != NO_VAL16)
+				desc->bitflags |= JOB_NTASKS_SET;
 			if (alloc && desc &&
 			    (desc->bitflags & JOB_NTASKS_SET)) {
 				if (desc->ntasks_per_node != NO_VAL16)
@@ -536,6 +545,12 @@ int main(int argc, char **argv)
 		list_iterator_destroy(iter_resp);
 		list_iterator_destroy(iter_req);
 	} else {
+		/*
+		 * Set JOB_NTASKS_SET to make SLURM_NTASKS get set when
+		 * --ntasks-per-node is requested.
+		 */
+		if (desc->ntasks_per_node != NO_VAL16)
+			desc->bitflags |= JOB_NTASKS_SET;
 		if (alloc && desc && (desc->bitflags & JOB_NTASKS_SET)) {
 			if (desc->ntasks_per_node != NO_VAL16)
 				desc->num_tasks =
@@ -678,6 +693,7 @@ relinquish:
 	}
 
 #ifdef MEMORY_LEAK_DEBUG
+	cli_filter_fini();
 	select_g_fini();
 	slurm_reset_all_options(&opt, false);
 	slurm_auth_fini();
@@ -979,7 +995,7 @@ static void _job_complete_handler(srun_job_complete_msg_t *comp)
 			}
 		}
 	} else {
-		verbose("%ps is finished.", &comp->step_id);
+		verbose("%ps is finished.", comp);
 	}
 }
 
