@@ -105,6 +105,9 @@ int main(int argc, char **argv)
 	slurm_init(NULL);
 	log_init(xbasename(argv[0]), logopt, 0, NULL);
 
+	if (cli_filter_init() != SLURM_SUCCESS)
+		fatal("failed to initialize cli_filter plugin");
+
 	_set_exit_code();
 	if (spank_init_allocator() < 0) {
 		error("Failed to initialize plugin stack");
@@ -344,6 +347,7 @@ int main(int argc, char **argv)
 		rc = _job_wait(resp->job_id);
 
 #ifdef MEMORY_LEAK_DEBUG
+	cli_filter_fini();
 	select_g_fini();
 	slurm_reset_all_options(&opt, false);
 	slurm_auth_fini();
@@ -379,8 +383,7 @@ static int _job_wait(uint32_t job_id)
 		rc = slurm_load_job(&resp, job_id, SHOW_ALL);
 		if (rc == SLURM_SUCCESS) {
 			for (i = 0, job_ptr = resp->job_array;
-			     (i < resp->record_count) && complete;
-			     i++, job_ptr++) {
+			     (i < resp->record_count); i++, job_ptr++) {
 				if (IS_JOB_FINISHED(job_ptr)) {
 					if (WIFEXITED(job_ptr->exit_code)) {
 						ec2 = WEXITSTATUS(job_ptr->
@@ -438,6 +441,10 @@ static int _fill_job_desc_from_opts(job_desc_msg_t *desc)
 		env_array_merge(&desc->environment, (const char **) environ);
 	} else if (!xstrcasecmp(opt.export_env, "ALL")) {
 		env_array_merge(&desc->environment, (const char **) environ);
+	} else if (!xstrcasecmp(opt.export_env, "NIL")) {
+		desc->environment = env_array_create();
+		env_array_merge_slurm(&desc->environment,
+				      (const char **)environ);
 	} else if (!xstrcasecmp(opt.export_env, "NONE")) {
 		desc->environment = env_array_create();
 		env_array_merge_slurm(&desc->environment,
