@@ -45,30 +45,34 @@
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+#include "src/interfaces/serializer.h"
 
-#include "src/slurmrestd/http_content_type.h"
-#include "src/slurmrestd/http_url.h"
-#include "src/slurmrestd/openapi.h"
 #include "src/slurmrestd/operations.h"
 #include "src/slurmrestd/rest_auth.h"
-#include "src/slurmrestd/xjson.h"
-#include "src/slurmrestd/xyaml.h"
 
+openapi_t *openapi_state = NULL;
 static pthread_rwlock_t paths_lock = PTHREAD_RWLOCK_INITIALIZER;
 static List paths = NULL;
 
 #define MAGIC 0xDFFEAAAE
+#define MAGIC_HEADER_ACCEPT 0xDF9EAABE
 
 typedef struct {
 	int magic;
 	/* unique tag per path */
 	int tag;
 	/* handler's callback to call on match */
-	operation_handler_t callback;
+	openapi_handler_t callback;
 	/* tag to hand to handler */
 	int callback_tag;
 } path_t;
 
+typedef struct {
+	int magic; /* MAGIC_HEADER_ACCEPT */
+	char *type; /* mime type and sub type unchanged */
+	float q; /* quality factor (priority) */
+} http_header_accept_t;
+
 static void _check_path_magic(const path_t *path)
 {
 	xassert(path->magic == MAGIC);
@@ -126,8 +130,7 @@ static int _match_path_key(void *x, void *ptr)
 }
 
 extern int bind_operation_handler(const char *str_path,
-				  operation_handler_t callback,
-				  int callback_tag)
+				  openapi_handler_t callback, int callback_tag)
 {
 	int path_tag;
 	path_t *path;
@@ -137,7 +140,7 @@ extern int bind_operation_handler(const char *str_path,
 	debug3("%s: binding %s to 0x%"PRIxPTR,
 	       __func__, str_path, (uintptr_t) callback);
 
-	path_tag = register_path_tag(str_path);
+	path_tag = register_path_tag(openapi_state, str_path);
 	if (path_tag == -1)
 		fatal_abort("%s: failure registering OpenAPI for path: %s",
 			    __func__, str_path);
@@ -146,7 +149,9 @@ extern int bind_operation_handler(const char *str_path,
 		goto exists;
 
 	/* add new path */
-	debug4("%s: new path %s with tag %d", __func__, str_path, path_tag);
+	debug4("%s: new path %s with path_tag %d callback_tag %d",
+	       __func__, str_path, path_tag, callback_tag);
+	print_path_tag_methods(openapi_state, path_tag);
 
 	path = xmalloc(sizeof(*path));
 	path->magic = MAGIC;
@@ -165,7 +170,7 @@ exists:
 static int _rm_path_callback(void *x, void *ptr)
 {
 	path_t *path = (path_t *)x;
-	operation_handler_t callback = ptr;
+	openapi_handler_t callback = ptr;
 
 	_check_path_magic(path);
 
@@ -174,12 +179,12 @@ static int _rm_path_callback(void *x, void *ptr)
 
 	debug5("%s: removing tag %d for callback %"PRIxPTR,
 	       __func__, path->tag, (uintptr_t) callback);
-	unregister_path_tag(path->tag);
+	unregister_path_tag(openapi_state, path->tag);
 
 	return 1;
 }
 
-extern int unbind_operation_handler(operation_handler_t callback)
+extern int unbind_operation_handler(openapi_handler_t callback)
 {
 	slurm_rwlock_wrlock(&paths_lock);
 
@@ -235,7 +240,7 @@ static int _resolve_path(on_http_request_args_t *args, int *path_tag,
 	/* attempt to identify path leaf types */
 	(void) data_convert_tree(path, DATA_TYPE_NONE);
 
-	*path_tag = find_path_tag(path, params, args->method);
+	*path_tag = find_path_tag(openapi_state, path, params, args->method);
 
 	FREE_NULL_DATA(path);
 
@@ -244,40 +249,30 @@ static int _resolve_path(on_http_request_args_t *args, int *path_tag,
 			args,
 			"Unable find requested URL. Please view /openapi/v3 for API reference.",
 			HTTP_STATUS_CODE_ERROR_NOT_FOUND, NULL);
-
-	return SLURM_SUCCESS;
+	else if (*path_tag == -2)
+		return _operations_router_reject(
+			args,
+			"Requested REST method is not defined at URL. Please view /openapi/v3 for API reference.",
+			HTTP_STATUS_CODE_ERROR_METHOD_NOT_ALLOWED, NULL);
+	else
+		return SLURM_SUCCESS;
 }
 
 static int _get_query(on_http_request_args_t *args, data_t **query,
-		      mime_types_t read_mime, mime_types_t write_mime)
+		      const char *read_mime)
 {
-	/* sanity check there is a HTML body to actually parse */
-	if ((read_mime == MIME_JSON || read_mime == MIME_YAML) &&
-	    args->body_length == 0)
-		return _operations_router_reject(
-			args, "Unable to parse empty HTML body.",
-			HTTP_STATUS_CODE_ERROR_BAD_REQUEST, NULL);
+	int rc = SLURM_SUCCESS;
 
 	/* post will have query in the body otherwise it is in the URL */
-	switch (read_mime) {
-	case MIME_JSON:
-		*query = parse_json(args->body, (args->body_length - 1));
-		break;
-	case MIME_URL_ENCODED:
-		/* everything but POST must be urlencoded */
-		if (args->method == HTTP_REQUEST_POST)
-			*query = parse_url_query(args->body, true);
-		else
-			*query = parse_url_query(args->query, true);
-		break;
-	case MIME_YAML:
-		*query = parse_yaml(args->body, args->body_length);
-		break;
-	default:
-		fatal_abort("%s: unknown read mime type", __func__);
-	}
+	if (args->method == HTTP_REQUEST_POST)
+		rc = serialize_g_string_to_data(query, args->body,
+						args->body_length, read_mime);
+	else
+		rc = serialize_g_string_to_data(
+			query, args->query,
+			(args->query ? strlen(args->query) : 0), read_mime);
 
-	if (!*query)
+	if (rc || !*query)
 		return _operations_router_reject(
 			args, "Unable to parse query.",
 			HTTP_STATUS_CODE_ERROR_BAD_REQUEST, NULL);
@@ -286,48 +281,145 @@ static int _get_query(on_http_request_args_t *args, data_t **query,
 
 }
 
-static int _resolve_mime(on_http_request_args_t *args, mime_types_t *read_mime,
-			 mime_types_t *write_mime)
+static void _parse_http_accept_entry(char *entry, List l)
 {
-	*read_mime = find_matching_mime_type(args->content_type);
+	char *save_ptr = NULL;
+	char *token = NULL;
+	char *buffer = xstrdup(entry);
+	http_header_accept_t *act = xmalloc(sizeof(*act));
+	act->magic = MAGIC_HEADER_ACCEPT;
+	act->type = NULL;
+	act->q = 1; /* default to 1 per rfc7231:5.3.1 */
+
+	token = strtok_r(buffer, ";", &save_ptr);
+
+	if (token) {
+		/* first token is the mime type */
+		xstrtrim(token);
+		act->type = xstrdup(token);
+	}
+	while ((token = strtok_r(NULL, ",", &save_ptr))) {
+		xstrtrim(token);
+		sscanf(token, "q=%f", &act->q);
+	}
+	xfree(buffer);
+
+	debug5("%s: found %s with q=%f", __func__, act->type, act->q);
+
+	list_append(l, act);
+}
+
+static int _compare_q(void *x, void *y)
+{
+	http_header_accept_t **xobj_ptr = x;
+	http_header_accept_t **yobj_ptr = y;
+	http_header_accept_t *xobj = *xobj_ptr;
+	http_header_accept_t *yobj = *yobj_ptr;
+
+	xassert(xobj->magic == MAGIC_HEADER_ACCEPT);
+	xassert(yobj->magic == MAGIC_HEADER_ACCEPT);
+
+	if (xobj->q < yobj->q)
+		return -1;
+	else if (xobj->q > yobj->q)
+		return 1;
+
+	return 0;
+}
+
+static void _http_accept_list_delete(void *x)
+{
+	http_header_accept_t *obj = (http_header_accept_t *) x;
+
+	if (!obj)
+		return;
+
+	xassert(obj->magic == MAGIC_HEADER_ACCEPT);
+	obj->magic = ~MAGIC_HEADER_ACCEPT;
+
+	xfree(obj->type);
+	xfree(obj);
+}
+
+static List _parse_http_accept(const char *accept)
+{
+	List l = list_create(_http_accept_list_delete);
+	xassert(accept);
+	char *save_ptr = NULL;
+	char *token = NULL;
+	char *buffer = xstrdup(accept);
+
+	token = strtok_r(buffer, ",", &save_ptr);
+	while (token) {
+		xstrtrim(token);
+		_parse_http_accept_entry(token, l);
+		token = strtok_r(NULL, ",", &save_ptr);
+	}
+	xfree(buffer);
+
+	list_sort(l, _compare_q);
+
+	return l;
+}
+
+static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
+			 const char **write_mime)
+{
+	*read_mime = args->content_type;
 
 	//TODO: check Content-encoding and make sure it is identity only!
 	//https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Encoding
 
-	if (*read_mime == MIME_UNKNOWN) {
-		*read_mime = MIME_URL_ENCODED;
+	if (!*read_mime) {
+		*read_mime = MIME_TYPE_URL_ENCODED;
 
 		debug4("%s: [%s] did not provide a known content type header. Assuming URL encoded.",
 		      __func__, args->context->con->name);
 	}
 
 	if (args->accept) {
-		List accept = parse_http_accept(args->accept);
+		List accept = _parse_http_accept(args->accept);
 		http_header_accept_t *ptr = NULL;
 		ListIterator itr = list_iterator_create(accept);
 		while ((ptr = list_next(itr))) {
+			xassert(ptr->magic == MAGIC_HEADER_ACCEPT);
+
 			debug4("%s: [%s] accepts %s with q=%f",
 			       __func__, args->context->con->name, ptr->type,
 			       ptr->q);
 
-			*write_mime = find_matching_mime_type(ptr->type);
-			if (*write_mime != MIME_UNKNOWN)
+			if ((*write_mime = resolve_mime_type(ptr->type))) {
+				debug4("%s: [%s] found accepts %s=%s with q=%f",
+				       __func__, args->context->con->name,
+				       ptr->type, *write_mime, ptr->q);
 				break;
+			} else {
+				debug4("%s: [%s] rejecting accepts %s with q=%f",
+				       __func__, args->context->con->name,
+				       ptr->type, ptr->q);
+			}
 		}
 		list_iterator_destroy(itr);
-		list_destroy(accept);
+		FREE_NULL_LIST(accept);
 	} else {
 		debug3("%s: [%s] Accept header not specified. Defaulting to JSON.",
 		       __func__, args->context->con->name);
-		*write_mime = MIME_JSON;
+		*write_mime = MIME_TYPE_JSON;
 	}
 
-	if (*write_mime == MIME_UNKNOWN)
+	if (!*write_mime)
 		return _operations_router_reject(
 			args, "Accept content type is unknown",
+			HTTP_STATUS_CODE_ERROR_UNSUPPORTED_MEDIA_TYPE, NULL);
+
+	if (args->method != HTTP_REQUEST_POST && args->body_length > 0)
+		return _operations_router_reject(
+			args,
+			"Unexpected http body provided for non-POST method",
 			HTTP_STATUS_CODE_ERROR_BAD_REQUEST, NULL);
 
-	if ((*read_mime != MIME_URL_ENCODED) && (args->body_length == 0)) {
+	if (xstrcasecmp(*read_mime, MIME_TYPE_URL_ENCODED) &&
+	    (args->body_length == 0)) {
 		/*
 		 * RFC7273#3.1.1.5 only specifies a sender SHOULD send
 		 * the correct content-type header but allows for them to be
@@ -339,59 +431,79 @@ static int _resolve_mime(on_http_request_args_t *args, mime_types_t *read_mime,
 		 */
 		debug("%s: [%s] Overriding content type from %s to %s for %s",
 		      __func__, args->context->con->name,
-		      get_mime_type_str(*read_mime),
-		      get_mime_type_str(MIME_URL_ENCODED),
+		      *read_mime,
+		      MIME_TYPE_URL_ENCODED,
 		      get_http_method_string(args->method));
 
-		*read_mime = MIME_URL_ENCODED;
+		*read_mime = MIME_TYPE_URL_ENCODED;
 	}
 
-	if (args->method != HTTP_REQUEST_POST && args->body_length > 0)
-		return _operations_router_reject(
-			args,
-			"Unexpected http body provided for non-POST method",
-			HTTP_STATUS_CODE_ERROR_BAD_REQUEST, NULL);
-
 	debug3("%s: [%s] mime read: %s write: %s",
-	       __func__, args->context->con->name,
-	       get_mime_type_str(*read_mime), get_mime_type_str(*write_mime));
-
+	       __func__, args->context->con->name, *read_mime, *write_mime);
 
 	return SLURM_SUCCESS;
 }
 
 static int _call_handler(on_http_request_args_t *args, data_t *params,
-			 data_t *query, operation_handler_t callback,
-			 int callback_tag, mime_types_t write_mime)
+			 data_t *query, openapi_handler_t callback,
+			 int callback_tag, const char *write_mime)
 {
 	int rc;
 	data_t *resp = data_new();
-	const char *body = NULL;
+	char *body = NULL;
+	http_status_code_t e;
+
+	debug3("%s: [%s] BEGIN: calling handler: 0x%"PRIXPTR"[%d] for path: %s",
+	       __func__, args->context->con->name, (uintptr_t) callback,
+	       callback_tag, args->path);
 
 	rc = callback(args->context->con->name, args->method, params, query,
 		      callback_tag, resp, args->context->auth);
 
-	if (data_get_type(resp) == DATA_TYPE_NULL)
-		/* no op */;
-	else if (write_mime == MIME_YAML)
-		body = dump_yaml(resp);
-	else if (write_mime == MIME_JSON)
-		body = dump_json(resp, DUMP_JSON_FLAGS_PRETTY);
-	else
-		fatal_abort("%s: unexpected mime type", __func__);
+	/*
+	 * Clear auth context after callback is complete. Client has to provide
+	 * full auth for every request already.
+	 */
+	FREE_NULL_REST_AUTH(args->context->auth);
 
-	if (rc) {
-		http_status_code_t e = HTTP_STATUS_CODE_SRVERR_INTERNAL;
+	if (data_get_type(resp) != DATA_TYPE_NULL) {
+		int rc2 = serialize_g_data_to_string(
+			&body, NULL, resp, write_mime, SER_FLAGS_PRETTY);
+
+		if (!rc)
+			rc = rc2;
+	}
+
+	if (rc == SLURM_NO_CHANGE_IN_DATA) {
+		/*
+		 * RFC#7232 Section:4.1
+		 *
+		 * Send minimal response that nothing has changed
+		 *
+		 */
+		send_http_response_args_t send_args = {
+			.con = args->context->con,
+			.http_major = args->http_major,
+			.http_minor = args->http_minor,
+			.status_code = HTTP_STATUS_CODE_REDIRECT_NOT_MODIFIED,
+		};
+		e = send_args.status_code;
+		rc = send_http_response(&send_args);
+	} else if (rc && (rc != ESLURM_REST_EMPTY_RESULT)) {
+		e = HTTP_STATUS_CODE_SRVERR_INTERNAL;
 
 		if (rc == ESLURM_REST_INVALID_QUERY)
-			e = HTTP_STATUS_CODE_ERROR_BAD_REQUEST;
+			e = HTTP_STATUS_CODE_ERROR_UNPROCESSABLE_CONTENT;
 		else if (rc == ESLURM_REST_FAIL_PARSING)
 			e = HTTP_STATUS_CODE_ERROR_BAD_REQUEST;
 		else if (rc == ESLURM_REST_INVALID_JOBS_DESC)
 			e = HTTP_STATUS_CODE_ERROR_BAD_REQUEST;
+		else if (rc == ESLURM_DATA_UNKNOWN_MIME_TYPE)
+			e = HTTP_STATUS_CODE_ERROR_UNSUPPORTED_MEDIA_TYPE;
+		else if (rc == ESLURM_INVALID_JOB_ID)
+			e = HTTP_STATUS_CODE_ERROR_NOT_FOUND;
 
-		rc = _operations_router_reject(args, body, e,
-					       get_mime_type_str(write_mime));
+		rc = _operations_router_reject(args, body, e, write_mime);
 	} else {
 		send_http_response_args_t send_args = {
 			.con = args->context->con,
@@ -405,12 +517,18 @@ static int _call_handler(on_http_request_args_t *args, data_t *params,
 		if (body) {
 			send_args.body = body;
 			send_args.body_length = strlen(body);
-			send_args.body_encoding = get_mime_type_str(write_mime);
+			send_args.body_encoding = write_mime;
 		}
 
 		rc = send_http_response(&send_args);
+		e = send_args.status_code;
 	}
 
+	debug3("%s: [%s] END: calling handler: (0x%"PRIXPTR") callback_tag %d for path: %s rc[%d]=%s status[%d]=%s",
+	       __func__, args->context->con->name, (uintptr_t) callback,
+	       callback_tag, args->path, rc, slurm_strerror(rc), e,
+	       get_http_status_code_string(e));
+
 	xfree(body);
 	FREE_NULL_DATA(resp);
 
@@ -424,16 +542,19 @@ extern int operations_router(on_http_request_args_t *args)
 	data_t *params = NULL;
 	int path_tag;
 	path_t *path = NULL;
-	operation_handler_t callback = NULL;
+	openapi_handler_t callback = NULL;
 	int callback_tag;
-	mime_types_t read_mime = MIME_UNKNOWN;
-	mime_types_t write_mime = MIME_UNKNOWN;
+	const char *read_mime = NULL;
+	const char *write_mime = NULL;
 
 	info("%s: [%s] %s %s",
 	     __func__, args->context->con->name,
 	     get_http_method_string(args->method), args->path);
 
 	if ((rc = rest_authenticate_http_request(args))) {
+		error("%s: [%s] authentication failed: %s",
+		      __func__, args->context->con->name,
+		      slurm_strerror(rc));
 		_operations_router_reject(args, "Authentication failure",
 					  HTTP_STATUS_CODE_ERROR_UNAUTHORIZED,
 					  NULL);
@@ -442,7 +563,7 @@ extern int operations_router(on_http_request_args_t *args)
 
 	params = data_set_dict(data_new());
 	if ((rc = _resolve_path(args, &path_tag, params)))
-	    return rc;
+		goto cleanup;
 
 	/*
 	 * Hold read lock while the callback is executing to avoid
@@ -466,7 +587,7 @@ extern int operations_router(on_http_request_args_t *args)
 	if ((rc = _resolve_mime(args, &read_mime, &write_mime)))
 		goto cleanup;
 
-	if ((rc = _get_query(args, &query, read_mime, write_mime)))
+	if ((rc = _get_query(args, &query, read_mime)))
 		goto cleanup;
 
 	rc = _call_handler(args, params, query, callback, callback_tag,
@@ -477,8 +598,7 @@ cleanup:
 	FREE_NULL_DATA(params);
 
 	/* always clear the auth context */
-	rest_auth_g_free(args->context->auth);
-	args->context->auth = NULL;
+	FREE_NULL_REST_AUTH(args->context->auth);
 
 	return rc;
 }
