@@ -509,7 +509,7 @@ static void _layout_resv_record(GtkTreeView *treeview,
 				int update)
 {
 	GtkTreeIter iter;
-	char time_buf[20];
+	char time_buf[256];
 	reserve_info_t *resv_ptr = sview_resv_info->resv_ptr;
 	char *temp_char = NULL;
 
@@ -626,7 +626,7 @@ static void _layout_resv_record(GtkTreeView *treeview,
 static void _update_resv_record(sview_resv_info_t *sview_resv_info_ptr,
 				GtkTreeStore *treestore)
 {
-	char tmp_duration[40], tmp_end[40], tmp_nodes[40], tmp_start[40];
+	char tmp_duration[40], tmp_end[256], tmp_nodes[40], tmp_start[256];
 	char tmp_cores[40], tmp_msd[40];
 	char *tmp_flags;
 	char *tmp_watts = NULL;
@@ -1255,6 +1255,8 @@ display_it:
 		   the return value */
 		create_treestore(tree_view, display_data_resv,
 				 SORTID_CNT, SORTID_TIME_START, SORTID_COLOR);
+
+		set_column_width_fixed(tree_view, SORTID_NODELIST, 100);
 	}
 
 	view = INFO_VIEW;
@@ -1281,7 +1283,7 @@ extern void specific_info_resv(popup_info_t *popup_win)
 	List resv_list = NULL;
 	List send_resv_list = NULL;
 	sview_resv_info_t *sview_resv_info_ptr = NULL;
-	int j=0, i=-1;
+	int j=0;
 	hostset_t hostset = NULL;
 	ListIterator itr = NULL;
 
@@ -1358,9 +1360,7 @@ display_it:
 	   the list */
 	send_resv_list = list_create(NULL);
 	itr = list_iterator_create(resv_list);
-	i = -1;
 	while ((sview_resv_info_ptr = list_next(itr))) {
-		i++;
 		resv_ptr = sview_resv_info_ptr->resv_ptr;
 		switch(spec_info->type) {
 		case PART_PAGE:
